"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = encode;

var _isType = _interopRequireDefault(require("../_internal/isType"));

var _utf8ToBytes = _interopRequireDefault(require("../_internal/utf8ToBytes"));

var _isArray = _interopRequireDefault(require("../lang/isArray"));

var isUint8Array = (0, _isType["default"])('Uint8Array');
var base64Chars = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/';
var paddingChar = '=';
var bitPadding = '00000000';

function octetToBase64(octets) {
  var len = octets.length;
  var bits = '';
  var paddingLen = 3 - len;
  var result = '';

  for (var i = 0; i < len; i++) {
    var bitStr = octets[i].toString(2);
    var pad = bitPadding.slice(0, 8 - bitStr.length);
    bits += pad + bitStr;
  }

  if (len === 1) {
    bits += bitPadding.slice(0, 4);
  } else if (len === 2) {
    bits += bitPadding.slice(0, 2);
  }

  for (var _i = 0, _len = bits.length / 6; _i < _len; _i++) {
    var sextet = parseInt(bits.slice(_i * 6, (_i + 1) * 6), 2);
    result += base64Chars[sextet];
  }

  while (paddingLen--) {
    result += paddingChar;
  }

  return result;
}

function encode(input) {
  var bytes = []; // TODO:
  //  This should use lang/isString method, but need to use type assertions.
  //  While the karma will throw errors if use type assertions.

  if (typeof input === 'string') {
    bytes = (0, _utf8ToBytes["default"])(input);
  } else if ((0, _isArray["default"])(input) || isUint8Array(input)) {
    bytes = input;
  } else {
    throw TypeError('the input parameter must be one of String, Byte Array, Uint8Array');
  }

  var result = '';

  for (var i = 0, len = Math.ceil(bytes.length / 3); i < len; i++) {
    // TODO:
    //  bytes.slice(i * 3, (i + 1) * 3) will be failed at type check
    result += octetToBase64(Array.prototype.slice.call(bytes, i * 3, (i + 1) * 3));
  }

  return result;
}

module.exports = exports.default;