function bytesToHex(bytes) {
    var hex = '';
    for (var i = 0; i < bytes.length; i++) {
        hex += ('0' + (bytes[i] & 0xff).toString(16)).slice(-2);
    }
    return hex;
}

function bytesToUtf8(bytes) {
    var codeUnits = [];
    var codePoint = 0;
    var utf8 = '';
    var byte = -1;
    for (var i = 0; i < bytes.length; i++) {
        byte = bytes[i];
        if (byte < 0x80) {
            codeUnits.push(bytes[i]);
        }
        else if (byte < 0xe0) {
            codePoint = ((bytes[i] & 31) << 6) + (bytes[++i] & 63);
            codeUnits.push(codePoint);
        }
        else if (byte < 0xf0) {
            codePoint =
                ((bytes[i] & 15) << 12) + ((bytes[++i] & 63) << 6) + (bytes[++i] & 63);
            codeUnits.push(codePoint);
        }
        else {
            codePoint =
                ((bytes[i] & 7) << 18) +
                    ((bytes[++i] & 63) << 12) +
                    ((bytes[++i] & 63) << 6) +
                    (bytes[++i] & 63);
            codePoint -= 0x10000;
            codeUnits.push((codePoint >> 10) + 0xd800, (codePoint % 0x400) + 0xdc00);
        }
        if (codeUnits.length > 0x3fff) {
            utf8 += String.fromCharCode.apply(null, codeUnits);
            codeUnits.length = 0;
        }
    }
    utf8 += String.fromCharCode.apply(null, codeUnits);
    return utf8;
}

var isType = function (type) {
    return function (o) {
        return Object.prototype.toString.call(o) === "[object " + type + "]";
    };
};

var isString = isType('String');

var base64Chars = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/';
var base64Map = {};
var bitPadding = '000000';
for (var i = 0; i < base64Chars.length; i++) {
    var ch = base64Chars.charAt(i);
    base64Map[ch] = i;
}
function base64ToBytes(str) {
    var bits = '';
    for (var i = 0; i < 4; i++) {
        var ch = str.charAt(i);
        var index = base64Map[ch];
        if (index > -1) {
            var bitStr = index.toString(2);
            var pad = bitPadding.slice(0, 6 - bitStr.length);
            bits += pad + bitStr;
        }
    }
    var bytes = [];
    for (var i = 0, len = Math.floor(bits.length / 8); i < len; i++) {
        bytes.push(parseInt(bits.slice(i * 8, (i + 1) * 8), 2));
    }
    return bytes;
}
function decode(input, options) {
    if (options === void 0) { options = {}; }
    if (!isString(input)) {
        throw TypeError('input parameter should be a string');
    }
    var _a = options.encoding, encoding = _a === void 0 ? 'utf8' : _a;
    var bytes = [];
    for (var i = 0, len = Math.ceil(input.length / 4); i < len; i++) {
        bytes.push.apply(bytes, base64ToBytes(input.slice(i * 4, (i + 1) * 4)));
    }
    if (encoding === 'binary') {
        return bytes;
    }
    else if (encoding === 'hex') {
        return bytesToHex(bytes);
    }
    return bytesToUtf8(bytes);
}

function decodeFile(input, options) {
    return new Promise(function (resolve, reject) {
        try {
            var type = options.type;
            var bytes = decode(input, { encoding: 'binary' });
            resolve(new Blob([Uint8Array.from(bytes)], { type: type }));
        }
        catch (err) {
            reject(err);
        }
    });
}

function utf8ToBytes(str) {
    var bytes = [];
    for (var i = 0; i < str.length; i++) {
        var code = str.charCodeAt(i);
        if (code < 0x007f) {
            bytes.push(code);
        }
        else if (code < 0x07ff) {
            var b1 = (code >> 6) | 192;
            var b2 = (code & 63) | 128;
            bytes.push(b1, b2);
        }
        else if (code >= 0xd800 && code <= 0xdbff) {
            var high = code;
            i++;
            var low = str.charCodeAt(i);
            code = 0x10000 + (high - 0xd800) * 0x400 + low - 0xdc00;
            var b1 = (code >> 18) | 240;
            var b2 = ((code >> 12) & 63) | 128;
            var b3 = ((code >> 6) & 63) | 128;
            var b4 = (code & 63) | 128;
            bytes.push(b1, b2, b3, b4);
        }
        else {
            var b1 = (code >> 12) | 224;
            var b2 = ((code >> 6) & 63) | 128;
            var b3 = (code & 63) | 128;
            bytes.push(b1, b2, b3);
        }
    }
    return bytes;
}

var isArray = isType('Array');

var isUint8Array = isType('Uint8Array');
var base64Chars$1 = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/';
var paddingChar = '=';
var bitPadding$1 = '00000000';
function octetToBase64(octets) {
    var len = octets.length;
    var bits = '';
    var paddingLen = 3 - len;
    var result = '';
    for (var i = 0; i < len; i++) {
        var bitStr = octets[i].toString(2);
        var pad = bitPadding$1.slice(0, 8 - bitStr.length);
        bits += pad + bitStr;
    }
    if (len === 1) {
        bits += bitPadding$1.slice(0, 4);
    }
    else if (len === 2) {
        bits += bitPadding$1.slice(0, 2);
    }
    for (var i = 0, len_1 = bits.length / 6; i < len_1; i++) {
        var sextet = parseInt(bits.slice(i * 6, (i + 1) * 6), 2);
        result += base64Chars$1[sextet];
    }
    while (paddingLen--) {
        result += paddingChar;
    }
    return result;
}
function encode(input) {
    var bytes = [];
    // TODO:
    //  This should use lang/isString method, but need to use type assertions.
    //  While the karma will throw errors if use type assertions.
    if (typeof input === 'string') {
        bytes = utf8ToBytes(input);
    }
    else if (isArray(input) || isUint8Array(input)) {
        bytes = input;
    }
    else {
        throw TypeError('the input parameter must be one of String, Byte Array, Uint8Array');
    }
    var result = '';
    for (var i = 0, len = Math.ceil(bytes.length / 3); i < len; i++) {
        // TODO:
        //  bytes.slice(i * 3, (i + 1) * 3) will be failed at type check
        result += octetToBase64(Array.prototype.slice.call(bytes, i * 3, (i + 1) * 3));
    }
    return result;
}

function encodeFile(file) {
    return new Promise(function (resolve, reject) {
        var reader = new FileReader();
        reader.addEventListener('load', function () {
            var str = reader.result
                ? reader.result.replace(/data:.+?,/, '')
                : '';
            resolve(str);
        });
        reader.addEventListener('error', function () {
            reject(new Error('error occurred in reading file'));
        });
        reader.readAsDataURL(file);
    });
}

var index = {
    decode: decode,
    decodeFile: decodeFile,
    encode: encode,
    encodeFile: encodeFile
};

var isAlphabetic = function (ch) {
    if (isString(ch) && ch.length === 1) {
        var charCode = ch.charCodeAt(0);
        return ((charCode >= 65 && charCode <= 90) || (charCode >= 97 && charCode <= 122));
    }
    return false;
};

var isAlphanumeric = function (ch) {
    if (isString(ch) && ch.length === 1) {
        var charCode = ch.charCodeAt(0);
        return ((charCode >= 48 && charCode <= 57) ||
            (charCode >= 65 && charCode <= 90) ||
            (charCode >= 97 && charCode <= 122));
    }
    return false;
};

// Todo: performance vs regexp
var isASCII = function (ch) {
    if (isString(ch) && ch.length === 1) {
        var charCode = ch.charCodeAt(0);
        return charCode <= 127;
    }
    return false;
};

var isCJK = function (ch) {
    if (isString(ch) && ch.length === 1) {
        return /[\u1100-\u11ff\u2e80-\u2fdf\u3040-\u318f\u3200-\u4dbf\u4e00-\u9fff\ua960-\ua97f\uf900-\ufaff\u3000-\u303f\uac00-\ud7af\ufe10-\ufe1f\ufe30-\ufe6f\uff00-\uff60\uffe0-\uffef]/.test(ch);
    }
    return false;
};

var isControl = function (ch) {
    if (isString(ch) && ch.length === 1) {
        var charCode = ch.charCodeAt(0);
        return (charCode >= 0 && charCode <= 31) || charCode === 127;
    }
    return false;
};

var isDigit = function (ch) {
    if (isString(ch) && ch.length === 1) {
        var charCode = ch.charCodeAt(0);
        return charCode >= 48 && charCode <= 57;
    }
    return false;
};

var isGraph = function (ch) {
    if (isString(ch) && ch.length === 1) {
        var charCode = ch.charCodeAt(0);
        return charCode >= 33 && charCode <= 126;
    }
    return false;
};

var isHexadecimal = function (ch) {
    if (isString(ch) && ch.length === 1) {
        var charCode = ch.charCodeAt(0);
        return ((charCode >= 48 && charCode <= 57) ||
            (charCode >= 65 && charCode <= 70) ||
            (charCode >= 97 && charCode <= 102));
    }
    return false;
};

var isLower = function (ch) {
    if (isString(ch) && ch.length === 1) {
        var charCode = ch.charCodeAt(0);
        return charCode >= 97 && charCode <= 122;
    }
    return false;
};

var isPrint = function (ch) {
    if (isString(ch) && ch.length === 1) {
        var charCode = ch.charCodeAt(0);
        return charCode >= 32 && charCode <= 126;
    }
    return false;
};

var isPunctuation = function (ch) {
    if (isString(ch) && ch.length === 1) {
        var charCode = ch.charCodeAt(0);
        return ((charCode >= 33 && charCode <= 47) ||
            (charCode >= 58 && charCode <= 64) ||
            (charCode >= 91 && charCode <= 96) ||
            (charCode >= 123 && charCode <= 126));
    }
    return false;
};

var isSpace = function (ch) {
    if (isString(ch) && ch.length === 1) {
        var charCode = ch.charCodeAt(0);
        return (charCode >= 9 && charCode <= 13) || charCode === 32;
    }
    return false;
};

var isUpper = function (ch) {
    if (isString(ch) && ch.length === 1) {
        var charCode = ch.charCodeAt(0);
        return charCode >= 65 && charCode <= 90;
    }
    return false;
};

var index$1 = {
    isAlphabetic: isAlphabetic,
    isAlphanumeric: isAlphanumeric,
    isASCII: isASCII,
    isCJK: isCJK,
    isControl: isControl,
    isDigit: isDigit,
    isGraph: isGraph,
    isHexadecimal: isHexadecimal,
    isLower: isLower,
    isPrint: isPrint,
    isPunctuation: isPunctuation,
    isSpace: isSpace,
    isUpper: isUpper
};

/*! *****************************************************************************
Copyright (c) Microsoft Corporation.

Permission to use, copy, modify, and/or distribute this software for any
purpose with or without fee is hereby granted.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
***************************************************************************** */

var __assign = function() {
    __assign = Object.assign || function __assign(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};

function hasOwn(obj, prop) {
    return Object.prototype.hasOwnProperty.call(obj, prop);
}

// Return a future date by the given string
function computeExpires(str) {
    var lastCh = str.charAt(str.length - 1);
    var value = parseInt(str, 10);
    var expires = new Date();
    switch (lastCh) {
        case 'Y':
            expires.setFullYear(expires.getFullYear() + value);
            break;
        case 'M':
            expires.setMonth(expires.getMonth() + value);
            break;
        case 'D':
            expires.setDate(expires.getDate() + value);
            break;
        case 'h':
            expires.setHours(expires.getHours() + value);
            break;
        case 'm':
            expires.setMinutes(expires.getMinutes() + value);
            break;
        case 's':
            expires.setSeconds(expires.getSeconds() + value);
            break;
        default:
            expires = new Date(str);
    }
    return expires;
}

// Convert an object to a cookie option string
function convert(opts) {
    var res = '';
    var key;
    for (key in opts) {
        /* istanbul ignore else */
        if (hasOwn(opts, key)) {
            if (/^expires$/i.test(key)) {
                var expires = opts[key];
                if (typeof expires !== 'object') {
                    expires += typeof expires === 'number' ? 'D' : '';
                    expires = computeExpires(String(expires));
                }
                res += ";" + key + "=" + expires.toUTCString();
            }
            else if (/^secure$/.test(key)) {
                if (opts[key]) {
                    res += ";" + key;
                }
            }
            else {
                res += ";" + key + "=" + String(opts[key]);
            }
        }
    }
    if (!hasOwn(opts, 'path')) {
        res += ';path=/';
    }
    return res;
}

function set(key, value, options, encoder) {
    if (options === void 0) { options = {}; }
    if (encoder === void 0) { encoder = encodeURIComponent; }
    var attrsStr = convert(options);
    var valueStr = typeof encoder === 'function' ? encoder(value) : value;
    var newCookie = key + "=" + valueStr + attrsStr;
    document.cookie = newCookie;
}

function remove(key, options) {
    var opts = { expires: -1 };
    if (options) {
        opts = __assign(__assign({}, options), opts);
    }
    set(key, 'a', opts);
}

function isEnabled() {
    var key = '@key@';
    var value = '1';
    var re = new RegExp("(?:^|; )" + key + "=" + value + "(?:;|$)");
    document.cookie = key + "=" + value + ";path=/;";
    var enabled = re.test(document.cookie);
    /* istanbul ignore else */
    if (enabled) {
        remove(key);
    }
    return enabled;
}

/**
 * Escape regexp special characters. The slash `/` will be escaped automatically.
 * http://stackoverflow.com/questions/494035/how-do-you-pass-a-variable-to-a-regular-expression-javascript/494122#494122
 */
function escapeRegexp(str) {
    return str.replace(/[.*+?^$|[\](){}\\-]/g, '\\$&');
}

function get(key, decoder) {
    if (decoder === void 0) { decoder = decodeURIComponent; }
    if (!isString(key) || !key) {
        return null;
    }
    var reKey = new RegExp("(?:^|; )" + escapeRegexp(key) + "(?:=([^;]*))?(?:;|$)");
    var match = reKey.exec(document.cookie);
    if (match === null) {
        return null;
    }
    return typeof decoder === 'function' ? decoder(match[1]) : match[1];
}

function getAll(decoder) {
    if (decoder === void 0) { decoder = decodeURIComponent; }
    var reKey = /(?:^|; )([^=]+?)(?:=([^;]*))?(?:;|$)/g;
    var cookies = {};
    var match;
    while ((match = reKey.exec(document.cookie))) {
        reKey.lastIndex = match.index + match.length - 1;
        cookies[match[1]] =
            typeof decoder === 'function'
                ? decoder(match[2])
                : /* istanbul ignore next */ match[2];
    }
    return cookies;
}

function getJson(key) {
    return get(key, JSON.parse);
}

function getRaw(key) {
    return get(key, null);
}

function setJson(key, value, options) {
    set(key, value, options, JSON.stringify);
}

function setRaw(key, value, options) {
    set(key, value, options, null);
}

var index$2 = {
    isEnabled: isEnabled,
    get: get,
    getAll: getAll,
    getJson: getJson,
    getRaw: getRaw,
    set: set,
    setJson: setJson,
    setRaw: setRaw,
    remove: remove
};

var isDate = isType('Date');

function diff(date1, date2, unit, roundFunc) {
    if (roundFunc === void 0) { roundFunc = Math.round; }
    var d1 = isDate(date1) ? date1 : new Date(date1);
    var d2 = isDate(date2) ? date2 : new Date(date2);
    var dateDiff = d2.getTime() - d1.getTime();
    var result = 0;
    switch (unit) {
        case 'Y':
            result = roundFunc(dateDiff / (365 * 24 * 60 * 60 * 1000));
            break;
        case 'M':
            result = roundFunc(dateDiff / (30 * 24 * 60 * 60 * 1000));
            break;
        case 'D':
            result = roundFunc(dateDiff / (24 * 60 * 60 * 1000));
            break;
        case 'h':
            result = roundFunc(dateDiff / (60 * 60 * 1000));
            break;
        case 'm':
            result = roundFunc(dateDiff / (60 * 1000));
            break;
        case 's':
            result = roundFunc(dateDiff / 1000);
            break;
        case 'ms':
        default:
            result = dateDiff;
            break;
    }
    return result;
}

var isNumber = isType('Number');

var padStart = function (str, len, chars) {
    if (str === void 0) { str = ''; }
    if (len === void 0) { len = 0; }
    if (chars === void 0) { chars = ' '; }
    var strValue = String(str);
    var charsValue = String(chars);
    var prefix = '';
    var strLen = strValue.length;
    var prefixLen = len - strLen;
    while (prefix.length < prefixLen) {
        prefix += charsValue;
    }
    prefix = prefix.slice(0, prefixLen);
    return prefix + strValue;
};

var dateFormat = function (date, format, isUTC) {
    if (!isNumber(date) && !isString(date) && !isDate(date)) {
        throw new Error('The first parameter should be a number, a parsable date string or a Date object');
    }
    var d;
    if (isNumber(date) || isString(date)) {
        d = new Date(date);
    }
    else {
        d = date;
    }
    if (!isString(format)) {
        return d.toString();
    }
    var year = isUTC ? d.getUTCFullYear() : d.getFullYear();
    var month = isUTC ? d.getUTCMonth() : d.getMonth();
    var day = isUTC ? d.getUTCDate() : d.getDate();
    var hour = isUTC ? d.getUTCHours() : d.getHours();
    var hour12 = hour % 12;
    var amOrPm = hour < 12 ? 'AM' : 'PM';
    var minute = isUTC ? d.getUTCMinutes() : d.getMinutes();
    var second = isUTC ? d.getUTCSeconds() : d.getSeconds();
    var millisecond = isUTC ? d.getUTCMilliseconds() : d.getMilliseconds();
    return format
        .replace(/(^|[^Y])YYYY([^Y]|$)/g, "$1" + year + "$2")
        .replace(/(^|[^Y])YY([^Y]|$)/g, "$1" + String(year).slice(-2) + "$2")
        .replace(/(^|[^M])MM([^M]|$)/g, "$1" + padStart(month + 1, 2, '0') + "$2")
        .replace(/(^|[^M])M([^M]|$)/g, "$1" + month + "$2")
        .replace(/(^|[^D])DD([^D]|$)/g, "$1" + padStart(day, 2, '0') + "$2")
        .replace(/(^|[^D])D([^D]|$)/g, "$1" + day + "$2")
        .replace(/(^|[^H])HH([^H]|$)/g, "$1" + padStart(hour, 2, '0') + "$2")
        .replace(/(^|[^H])H([^H]|$)/g, "$1" + hour + "$2")
        .replace(/(^|[^h])hh([^h]|$)/g, "$1" + padStart(hour12, 2, '0') + "$2")
        .replace(/(^|[^h])h([^h]|$)/g, "$1" + hour12 + "$2")
        .replace(/(^|[^A])A([^A]|$)/g, "$1" + amOrPm + "$2")
        .replace(/(^|[^a])a([^a]|$)/g, "$1" + amOrPm.toLowerCase() + "$2")
        .replace(/(^|[^m])mm([^m]|$)/g, "$1" + padStart(minute, 2, '0') + "$2")
        .replace(/(^|[^m])m([^m]|$)/g, "$1" + minute + "$2")
        .replace(/(^|[^s])ss([^s]|$)/g, "$1" + padStart(second, 2, '0') + "$2")
        .replace(/(^|[^s])s([^s]|$)/g, "$1" + second + "$2")
        .replace(/(^|[^S]+)([S]+)([^S]+|$)/g, function (match, s1, s2, s3) {
        var msStr = padStart(String(millisecond), 3, '0');
        for (var i = 3; i < s2.length; i++) {
            msStr += '0';
        }
        msStr = msStr.slice(0, s2.length);
        return "" + s1 + msStr + s3;
    });
};

function parse(date, isUTC) {
    if (isUTC === void 0) { isUTC = false; }
    if (!isNumber(date) && !isString(date) && !isDate(date)) {
        throw new Error('The first parameter should be a number, a parsable date string or a Date object');
    }
    var d;
    if (isNumber(date) || isString(date)) {
        d = new Date(date);
    }
    else {
        d = date;
    }
    return {
        years: isUTC ? d.getUTCFullYear() : d.getFullYear(),
        months: isUTC ? d.getUTCMonth() : d.getMonth(),
        dates: isUTC ? d.getUTCDate() : d.getDate(),
        hours: isUTC ? d.getUTCHours() : d.getHours(),
        minutes: isUTC ? d.getUTCMinutes() : d.getMinutes(),
        seconds: isUTC ? d.getUTCSeconds() : d.getSeconds(),
        milliseconds: isUTC ? d.getUTCMilliseconds() : d.getMilliseconds()
    };
}

var locale = [
    ['just now', 'right now'],
    ['%s seconds ago', 'in %s seconds'],
    ['1 minute ago', 'in 1 minute'],
    ['%s minutes ago', 'in %s minutes'],
    ['1 hour ago', 'in 1 hour'],
    ['%s hours ago', 'in %s hours'],
    ['1 day ago', 'in 1 day'],
    ['%s days ago', 'in %s days'],
    ['1 week ago', 'in 1 week'],
    ['%s weeks ago', 'in %s weeks'],
    ['1 month ago', 'in 1 month'],
    ['%s months ago', 'in %s months'],
    ['1 year ago', 'in 1 year'],
    ['%s years ago', 'in %s years'],
];

var locale$1 = [
    ['刚刚', '片刻后'],
    ['%s 秒前', '%s 秒后'],
    ['1 分钟前', '1 分钟后'],
    ['%s 分钟前', '%s 分钟后'],
    ['1 小时前', '1 小时后'],
    ['%s 小时前', '%s 小时后'],
    ['1 天前', '1 天后'],
    ['%s 天前', '%s 天后'],
    ['1 周前', '1 周后'],
    ['%s 周前', '%s 周后'],
    ['1 个月前', '1 个月后'],
    ['%s 个月前', '%s 个月后'],
    ['1 年前', '1 年后'],
    ['%s 年前', '%s 年后'],
];

var locales = {
    en_US: locale,
    zh_CN: locale$1
};
var currentLocale = 'en_US';
var timeAgo = {
    getLocale: function () {
        return currentLocale;
    },
    setLocale: function (name) {
        /* istanbul ignore else */
        if (hasOwn(locales, name)) {
            currentLocale = name;
        }
    },
    addLocale: function (name, locale) {
        locales[name] = locale;
    },
    format: function (date, locale, nowDate) {
        if (locale === void 0) { locale = currentLocale; }
        if (nowDate === void 0) { nowDate = new Date(); }
        var d = new Date(date);
        var actualLocale = hasOwn(locales, locale) ? locale : currentLocale;
        var sign = nowDate.getTime() - d.getTime() >= 0 ? 1 : -1;
        var diff = Math.abs(nowDate.getTime() - d.getTime());
        var years = Math.floor(diff / (365 * 24 * 60 * 60 * 1000));
        var months = Math.floor(diff / (30 * 24 * 60 * 60 * 1000));
        var weeks = Math.floor(diff / (7 * 24 * 60 * 60 * 1000));
        var days = Math.floor(diff / (24 * 60 * 60 * 1000));
        var hours = Math.floor(diff / (60 * 60 * 1000));
        var minutes = Math.floor(diff / (60 * 1000));
        var seconds = Math.floor(diff / 1000);
        var index = 0;
        var number = 0;
        if (seconds === 0) {
            index = 0;
            number = 0;
        }
        else if (seconds >= 1 && minutes === 0) {
            index = 1;
            number = seconds;
        }
        else if (minutes === 1) {
            index = 2;
            number = minutes;
        }
        else if (minutes > 1 && hours === 0) {
            index = 3;
            number = minutes;
        }
        else if (hours === 1) {
            index = 4;
            number = hours;
        }
        else if (hours > 1 && days === 0) {
            index = 5;
            number = hours;
        }
        else if (days === 1) {
            index = 6;
            number = days;
        }
        else if (days > 1 && weeks === 0) {
            index = 7;
            number = days;
        }
        else if (weeks === 1) {
            index = 8;
            number = weeks;
        }
        else if (weeks > 1 && months === 0) {
            index = 9;
            number = weeks;
        }
        else if (months === 1) {
            index = 10;
            number = months;
        }
        else if (months > 1 && years === 0) {
            index = 11;
            number = months;
        }
        else if (years === 1) {
            index = 12;
            number = years;
        }
        else {
            index = 13;
            number = years;
        }
        var tmplPair = locales[actualLocale][index];
        var tmpl = sign > 0 ? tmplPair[0] : tmplPair[1];
        return tmpl.replace(/%s/, String(number));
    }
};

var index$3 = {
    diff: diff,
    format: dateFormat,
    parse: parse,
    timeAgo: timeAgo
};

function isElement(obj) {
    return !!(obj && obj.nodeType === 1);
}

function pageX(el) {
    // TODO: to throw a type error?
    if (!isElement(el))
        return 0;
    var parent = el;
    var x = 0;
    while (parent) {
        x += parent.offsetLeft;
        parent = parent.offsetParent;
    }
    return x;
}

function isWindow(obj) {
    return !!(obj && obj.self === obj);
}

function scrollX(elOrWindow) {
    if (elOrWindow === void 0) { elOrWindow = window; }
    var x = 0;
    if (isWindow(elOrWindow)) {
        var win = elOrWindow;
        x =
            win.scrollX || win.pageXOffset || win.document.documentElement.scrollLeft;
    }
    else if (isElement(elOrWindow)) {
        x = elOrWindow.scrollLeft;
    }
    return x;
}

function clientX(el) {
    var x = 0;
    if (isElement(el)) {
        x = pageX(el) - scrollX();
    }
    return x;
}

function pageY(el) {
    // TODO: to throw a type error?
    if (!isElement(el))
        return 0;
    var parent = el;
    var y = 0;
    while (parent) {
        y += parent.offsetTop;
        parent = parent.offsetParent;
    }
    return y;
}

function scrollY(elOrWindow) {
    if (elOrWindow === void 0) { elOrWindow = window; }
    var y = 0;
    if (isWindow(elOrWindow)) {
        var win = elOrWindow;
        y = win.scrollY || win.pageYOffset || win.document.documentElement.scrollTop;
    }
    else if (isElement(elOrWindow)) {
        y = elOrWindow.scrollTop;
    }
    return y;
}

function clientY(el) {
    var y = 0;
    if (isElement(el)) {
        y = pageY(el) - scrollY();
    }
    return y;
}

function createEvent(type, options) {
    if (options === void 0) { options = {}; }
    var _a = options.bubbles, bubbles = _a === void 0 ? false : _a, _b = options.cancelable, cancelable = _b === void 0 ? false : _b, _c = options.detail, detail = _c === void 0 ? null : _c;
    var event;
    try {
        event = new CustomEvent(type, options);
    }
    catch (err) {
        /* istanbul ignore next */
        // @ts-ignore
        event = document.createEvent('Event');
        /* istanbul ignore next */
        // @ts-ignore
        if (typeof event.initCustomEvent === 'function') {
            event.initCustomEvent(type, bubbles, cancelable, detail);
        }
        else {
            event.initEvent(type, bubbles, cancelable);
            // @ts-ignore
            event.detail = detail;
        }
    }
    return event;
}

/**
 * domready (c) Dustin Diaz 2014 - License MIT
 */
var fns = [];
var doc = typeof document === 'object' && document;
// @ts-ignore
var hack = doc && doc.documentElement.doScroll;
var domContentLoaded = 'DOMContentLoaded';
var loaded = doc &&
    (hack ? /* istanbul ignore next */ /^loaded|^c/ : /^loaded|^i|^c/).test(doc.readyState);
var listener;
/* istanbul ignore else */
if (!loaded && doc) {
    listener = function () {
        var fn = fns.shift();
        doc.removeEventListener(domContentLoaded, listener);
        loaded = true;
        while (fn) {
            fn();
            fn = fns.shift();
        }
    };
    doc.addEventListener(domContentLoaded, listener);
}
function domReady(fn) {
    if (loaded) {
        setTimeout(fn, 0);
    }
    else {
        fns.push(fn);
    }
}

function insertScript(url, options) {
    if (options === void 0) { options = {}; }
    var script = document.createElement('script');
    script.src = url;
    Object.keys(options).forEach(function (key) {
        // @ts-ignore
        script[key] = options[key];
    });
    document.body.appendChild(script);
    return script;
}

// TODO: default value is window
function viewport(elOrWindow) {
    var width = 0;
    var height = 0;
    if (isWindow(elOrWindow)) {
        var win = elOrWindow;
        width =
            win.innerWidth ||
                /* istanbul ignore next */ win.document.documentElement.clientWidth;
        height =
            win.innerHeight ||
                /* istanbul ignore next */ win.document.documentElement.clientHeight;
    }
    else if (isElement(elOrWindow)) {
        var el = elOrWindow;
        width = el.clientWidth;
        height = el.clientHeight;
    }
    return { width: width, height: height };
}

function isInViewport(el) {
    if (!isElement(el))
        return false;
    var _a = el.getBoundingClientRect(), top = _a.top, left = _a.left, right = _a.right, bottom = _a.bottom;
    var _b = viewport(window), width = _b.width, height = _b.height;
    return !(top < 0 || bottom > height || left < 0 || right > width);
}

function pageHeight(el) {
    if (!el) {
        return document.documentElement.scrollHeight;
    }
    if (!isElement(el)) {
        throw new TypeError('the parameter el must be an valid html element');
    }
    return el.scrollHeight;
}

function pageWidth(el) {
    if (!el) {
        return document.documentElement.scrollWidth;
    }
    if (!isElement(el)) {
        throw new TypeError('the parameter el must be an valid html element');
    }
    return el.scrollWidth;
}

var isObject = isType('Object');

var isFunction = function (o) { return typeof o === 'function'; };

/**
 * https://github.com/gre/bezier-easing
 * BezierEasing - use bezier curve for transition easing function
 * by Gaëtan Renaudeau 2014 - 2015 – MIT License
 */
// These values are established by empiricism with tests (tradeoff: performance VS precision)
var NEWTON_ITERATIONS = 4;
var NEWTON_MIN_SLOPE = 0.001;
var SUBDIVISION_PRECISION = 0.0000001;
var SUBDIVISION_MAX_ITERATIONS = 10;
var kSplineTableSize = 11;
var kSampleStepSize = 1.0 / (kSplineTableSize - 1.0);
var float32ArraySupported = typeof Float32Array === 'function';
function A(aA1, aA2) {
    return 1.0 - 3.0 * aA2 + 3.0 * aA1;
}
function B(aA1, aA2) {
    return 3.0 * aA2 - 6.0 * aA1;
}
function C(aA1) {
    return 3.0 * aA1;
}
// Returns x(t) given t, x1, and x2, or y(t) given t, y1, and y2.
function calcBezier(aT, aA1, aA2) {
    return ((A(aA1, aA2) * aT + B(aA1, aA2)) * aT + C(aA1)) * aT;
}
// Returns dx/dt given t, x1, and x2, or dy/dt given t, y1, and y2.
function getSlope(aT, aA1, aA2) {
    return 3.0 * A(aA1, aA2) * aT * aT + 2.0 * B(aA1, aA2) * aT + C(aA1);
}
function binarySubdivide(aX, aA, aB, mX1, mX2) {
    var currentX;
    var currentT;
    var i = 0;
    do {
        currentT = aA + (aB - aA) / 2.0;
        currentX = calcBezier(currentT, mX1, mX2) - aX;
        if (currentX > 0.0) {
            aB = currentT;
        }
        else {
            aA = currentT;
        }
    } while (Math.abs(currentX) > SUBDIVISION_PRECISION &&
        ++i < SUBDIVISION_MAX_ITERATIONS);
    return currentT;
}
function newtonRaphsonIterate(aX, aGuessT, mX1, mX2) {
    for (var i = 0; i < NEWTON_ITERATIONS; ++i) {
        var currentSlope = getSlope(aGuessT, mX1, mX2);
        if (currentSlope === 0.0) {
            return aGuessT;
        }
        var currentX = calcBezier(aGuessT, mX1, mX2) - aX;
        aGuessT -= currentX / currentSlope;
    }
    return aGuessT;
}
function LinearEasing(x) {
    return x;
}
function bezier(mX1, mY1, mX2, mY2) {
    if (!(mX1 >= 0 && mX1 <= 1 && mX2 >= 0 && mX2 <= 1)) {
        throw new Error('bezier x values must be in [0, 1] range');
    }
    if (mX1 === mY1 && mX2 === mY2) {
        return LinearEasing;
    }
    // Precompute samples table
    var sampleValues = float32ArraySupported
        ? new Float32Array(kSplineTableSize)
        : new Array(kSplineTableSize);
    for (var i = 0; i < kSplineTableSize; ++i) {
        sampleValues[i] = calcBezier(i * kSampleStepSize, mX1, mX2);
    }
    function getTForX(aX) {
        var intervalStart = 0.0;
        var currentSample = 1;
        var lastSample = kSplineTableSize - 1;
        for (; currentSample !== lastSample && sampleValues[currentSample] <= aX; ++currentSample) {
            intervalStart += kSampleStepSize;
        }
        --currentSample;
        // Interpolate to provide an initial guess for t
        var dist = (aX - sampleValues[currentSample]) /
            (sampleValues[currentSample + 1] - sampleValues[currentSample]);
        var guessForT = intervalStart + dist * kSampleStepSize;
        var initialSlope = getSlope(guessForT, mX1, mX2);
        if (initialSlope >= NEWTON_MIN_SLOPE) {
            return newtonRaphsonIterate(aX, guessForT, mX1, mX2);
        }
        else if (initialSlope === 0.0) {
            return guessForT;
        }
        else {
            return binarySubdivide(aX, intervalStart, intervalStart + kSampleStepSize, mX1, mX2);
        }
    }
    return function BezierEasing(x) {
        // Because JavaScript number are imprecise, we should guarantee the extremes are right.
        if (x === 0) {
            return 0;
        }
        if (x === 1) {
            return 1;
        }
        return calcBezier(getTForX(x), mY1, mY2);
    };
}

var linear = bezier(0, 0, 1, 1);

var ease = bezier(0.25, 0.1, 0.25, 1);

var easeIn = bezier(0.42, 0, 1, 1);

var easeInOut = bezier(0.42, 0, 0.58, 1);

var easeOut = bezier(0, 0, 0.58, 1);

var isBrowser = typeof window !== 'undefined' && typeof window.document !== 'undefined';

var now;
if (isBrowser && window.performance && window.performance.now) {
    now = function () { return performance.now(); };
}
else {
    var loadTime_1 = Date.now();
    now = function () { return Date.now() - loadTime_1; };
}
var now$1 = now;

var vendors = ['webkit', 'moz'];
var root = isBrowser ? window : global;
var name = 'AnimationFrame';
// @ts-ignore
var raf = root["request" + name];
// @ts-ignore
var caf = root["cancel" + name];
for (var i$1 = 0; i$1 < vendors.length && !raf; i$1++) {
    // @ts-ignore
    raf = root[vendors[i$1] + "Request" + name];
    // @ts-ignore
    caf = root[vendors[i$1] + "Cancel" + name];
}
if (!raf || !caf) {
    var queue_1 = [];
    var tick_1 = 1000 / 60;
    var lastTime_1 = 0;
    var id_1 = 0;
    raf = function (callback) {
        if (queue_1.length === 0) {
            var nowTime = now$1();
            var next = Math.max(0, tick_1 - (nowTime - lastTime_1));
            lastTime_1 = nowTime + next;
            setTimeout(function () {
                var copiedQueue = queue_1.slice();
                queue_1.length = 0;
                for (var i = 0; i < copiedQueue.length; i++) {
                    if (!copiedQueue[i].cancelled) {
                        try {
                            copiedQueue[i].callback(lastTime_1);
                        }
                        catch (err) {
                            setTimeout(function () {
                                throw err;
                            }, 0);
                        }
                    }
                }
            }, Math.round(next));
        }
        var timerId = id_1++;
        queue_1.push({
            timerId: timerId,
            callback: callback,
            cancelled: false
        });
        return timerId;
    };
    caf = function (timerId) {
        for (var i = 0; i < queue_1.length; i++) {
            if (queue_1[i].timerId === timerId) {
                queue_1[i].cancelled = true;
            }
        }
    };
}
function exposeRaf(fn) {
    return raf.call(root, fn);
}
exposeRaf.cancel = function (timerId) {
    caf.call(root, timerId);
};

var easings = {
    linear: linear,
    ease: ease,
    'ease-in': easeIn,
    'ease-in-out': easeInOut,
    'ease-out': easeOut
};
var reEasingKeyword = /^(linear|ease|ease-in|ease-in-out|ease-out)$/;
var reCubicBezier = /^cubic-bezier\(\s*(\d+|\d+\.\d+|\.\d+)\s*,\s*(\d+|\d+\.\d+|\.\d+)\s*,\s*(\d+|\d+\.\d+|\.\d+)\s*,\s*(\d+|\d+\.\d+|\.\d+)\s*\)$/;
function scrollTo(elOrWindow, x, y) {
    if (x === void 0) { x = 0; }
    if (y === void 0) { y = 0; }
    var el = window;
    var options = {};
    if (isElement(elOrWindow) || isWindow(elOrWindow)) {
        el = elOrWindow;
        if (isObject(x)) {
            options = x;
        }
        else {
            options.x = x;
            options.y = y;
        }
    }
    else {
        if (isObject(elOrWindow)) {
            options = elOrWindow;
        }
        else {
            options.x = elOrWindow || 0;
            options.y = x;
        }
    }
    if (!hasOwn(options, 'x'))
        options.x = options.left || 0;
    if (!hasOwn(options, 'y'))
        options.y = options.top || 0;
    var innerScrollTo = function (opts) {
        var left = opts.x, top = opts.y, behavior = opts.behavior;
        if (isFunction(el.scrollTo)) {
            if (behavior) {
                el.scrollTo({ left: left, top: top, behavior: behavior });
            }
            else {
                el.scrollTo(left, top);
            }
        }
        else {
            /* istanbul ignore else */
            if (left !== undefined) {
                el.scrollLeft = left;
            }
            /* istanbul ignore else */
            if (top !== undefined) {
                el.scrollTop = top;
            }
        }
    };
    if (options.easing) {
        var easingFn_1;
        if (reEasingKeyword.test(options.easing)) {
            easingFn_1 = easings[options.easing];
        }
        else if (reCubicBezier.test(options.easing)) {
            var matched = options.easing.match(reCubicBezier);
            var x1 = 0;
            var y1 = 0;
            var x2 = 0;
            var y2 = 0;
            /* istanbul ignore else */
            if (matched) {
                x1 = Number(matched[1]);
                y1 = Number(matched[2]);
                x2 = Number(matched[3]);
                y2 = Number(matched[4]);
            }
            easingFn_1 = bezier(x1, y1, x2, y2);
        }
        else {
            innerScrollTo(options);
            return;
        }
        var minDuration = 150;
        var maxDuration = 500;
        var startX_1 = scrollX(el);
        var startY_1 = scrollY(el);
        // @ts-ignore
        var diffX_1 = options.x - startX_1;
        // @ts-ignore
        var diffY_1 = options.y - startY_1;
        var diff = Math.max(Math.abs(diffX_1), Math.abs(diffY_1));
        var duration = Math.min(minDuration + 100 * (diff / 1000), maxDuration);
        var tickTime = 1000 / 16;
        var tickTotal_1 = Math.ceil(duration / tickTime);
        var nextX_1 = startX_1;
        var nextY_1 = startY_1;
        var tickCount_1 = 0;
        var runScroll_1 = function (time) {
            if (tickCount_1 <= tickTotal_1) {
                nextX_1 = startX_1 + diffX_1 * Math.max(easingFn_1(tickCount_1 / tickTotal_1), 1);
                nextY_1 = startY_1 + diffY_1 * Math.max(easingFn_1(tickCount_1 / tickTotal_1), 1);
                innerScrollTo({ x: nextX_1, y: nextY_1 });
                tickCount_1++;
                exposeRaf(runScroll_1);
            }
        };
        exposeRaf(runScroll_1);
    }
    else {
        innerScrollTo(options);
    }
}

var index$4 = {
    clientX: clientX,
    clientY: clientY,
    createEvent: createEvent,
    domReady: domReady,
    insertScript: insertScript,
    isElement: isElement,
    isInViewport: isInViewport,
    isWindow: isWindow,
    pageX: pageX,
    pageY: pageY,
    pageHeight: pageHeight,
    pageWidth: pageWidth,
    scrollTo: scrollTo,
    scrollX: scrollX,
    scrollY: scrollY,
    viewport: viewport
};

var index$5 = {
    cubicBezier: bezier,
    ease: ease,
    easeIn: easeIn,
    easeInOut: easeInOut,
    easeOut: easeOut,
    linear: linear
};

function flattenEvents(events) {
    var listeners = [];
    for (var type in events) {
        /* istanbul ignore else */
        if (hasOwn(events, type)) {
            listeners = listeners.concat(events[type]);
        }
    }
    return listeners;
}

var EVENTS_KEY = '@events';
var Emitter = /** @class */ (function () {
    function Emitter() {
        this[EVENTS_KEY] = {};
    }
    // Add a listener for a given event.
    Emitter.prototype.on = function (type, fn) {
        (this[EVENTS_KEY][type] = this[EVENTS_KEY][type] || []).push(fn);
    };
    // Add a listener for a given event.
    // This listener can be called once, then will be removed.
    Emitter.prototype.once = function (type, fn) {
        var wrapped = function wrapped() {
            var args = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                args[_i] = arguments[_i];
            }
            // @ts-ignore
            return fn.apply(this, args);
        };
        wrapped.$once = true;
        this.on(type, wrapped);
    };
    // Remove all event listeners
    // or remove the given event listeners
    // or remove the specified listener for the given event.
    Emitter.prototype.off = function (type, fn) {
        // @ts-ignore
        var events = this[EVENTS_KEY][type];
        if (arguments.length === 0) {
            this[EVENTS_KEY] = {};
        }
        else if (arguments.length === 1) {
            // @ts-ignore
            delete this[EVENTS_KEY][type];
        }
        else {
            /* istanbul ignore else */
            if (events) {
                var len = events.length;
                var i = 0;
                var listener = void 0;
                while (i < len) {
                    listener = events[i];
                    if (listener === fn) {
                        events.splice(i, 1);
                        len--;
                    }
                    else {
                        i++;
                    }
                }
                if (!events.length) {
                    // @ts-ignore
                    delete this[EVENTS_KEY][type];
                }
            }
        }
    };
    // Trigger a given event with optional arguments.
    Emitter.prototype.emit = function (type) {
        var args = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            args[_i - 1] = arguments[_i];
        }
        var listeners = this[EVENTS_KEY][type];
        if (!listeners)
            return;
        var shallowListeners = listeners.slice();
        for (var i = 0; i < shallowListeners.length; i++) {
            var listener = shallowListeners[i];
            // @ts-ignore
            if (listener.$once) {
                this.off(type, listener);
            }
            listener.apply(this, args);
        }
    };
    // Get the event listeners or all event listeners.
    Emitter.prototype.getListeners = function (type) {
        if (type) {
            return this[EVENTS_KEY][type] || [];
        }
        return flattenEvents(this[EVENTS_KEY]);
    };
    return Emitter;
}());

function create() {
    return new Emitter();
}

var globalEmitter = create();

var emit = globalEmitter.emit.bind(globalEmitter);

var getListeners = globalEmitter.getListeners.bind(globalEmitter);

var off = globalEmitter.off.bind(globalEmitter);

var on = globalEmitter.on.bind(globalEmitter);

var once = globalEmitter.once.bind(globalEmitter);

var index$6 = {
    Emitter: Emitter,
    create: create,
    emit: emit,
    getListeners: getListeners,
    off: off,
    on: on,
    once: once
};

function getType(o) {
    return Object.prototype.toString.call(o).slice(8, -1);
}

var isBoolean = isType('Boolean');

var isError = isType('Error');

var isNil = function (o) { return o === null || o === undefined; };

var isNull = function (o) { return o === null; };

var isRegExp = isType('RegExp');

var isUndefined = function (o) { return o === undefined; };

var index$7 = {
    getType: getType,
    isArray: isArray,
    isBoolean: isBoolean,
    isDate: isDate,
    isError: isError,
    isFunction: isFunction,
    isNil: isNil,
    isNull: isNull,
    isNumber: isNumber,
    isObject: isObject,
    isRegExp: isRegExp,
    isString: isString,
    isUndefined: isUndefined
};

var basename = function (path, ext, sep) {
    var pathString = String(path);
    var lastSepIndex = pathString.lastIndexOf(sep || '/');
    var start = lastSepIndex > -1 ? lastSepIndex + 1 : 0;
    var end = pathString.length - (ext || '').length;
    return pathString.slice(start, end);
};

var dirname = function (path, sep) {
    var pathString = String(path);
    var lastSepIndex = pathString.lastIndexOf(sep || '/');
    var end = lastSepIndex > -1 ? lastSepIndex : 0;
    return pathString.slice(0, end);
};

var extname = function (path) {
    var pathString = String(path);
    var lastDotIndex = pathString.lastIndexOf('.');
    var ext = '';
    if (lastDotIndex > -1) {
        ext = pathString.slice(lastDotIndex);
    }
    return ext;
};

var join = function () {
    var args = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        args[_i] = arguments[_i];
    }
    if (args.length === 0)
        return '';
    return args.join('/').replace(/[/]+/g, '/');
};

var index$8 = {
    basename: basename,
    dirname: dirname,
    extname: extname,
    join: join
};

function parse$1(search) {
    if (search === void 0) { search = ''; }
    var queryString = search.charAt(0) === '?' ? search.slice(1) : search;
    var reArrayName = /^[\w\W]+\[\]$/;
    var query = {};
    queryString.split('&').forEach(function (part) {
        if (part) {
            var equalSignIndex = part.indexOf('=');
            var nameString = equalSignIndex > -1 ? part.slice(0, equalSignIndex) : part;
            var valueString = equalSignIndex > -1 ? part.slice(equalSignIndex + 1) : '';
            var name_1 = decodeURIComponent(nameString);
            var value = decodeURIComponent(valueString);
            if (reArrayName.test(name_1)) {
                name_1 = name_1.slice(0, -2);
                if (!query[name_1]) {
                    query[name_1] = [value];
                    return;
                }
            }
            if (query[name_1]) {
                if (!Array.isArray(query[name_1])) {
                    query[name_1] = [query[name_1]];
                }
                query[name_1].push(value);
            }
            else {
                query[name_1] = value;
            }
        }
    });
    return query;
}

function get$1(name, search) {
    var query = parse$1(search);
    return query[name];
}

function stringify(query) {
    var pairs = [];
    /* istanbul ignore else */
    if (isObject(query)) {
        var _loop_1 = function (name_1) {
            /* istanbul ignore else */
            if (hasOwn(query, name_1)) {
                var value = query[name_1];
                var encodedName_1 = encodeURIComponent(name_1);
                if (Array.isArray(value)) {
                    value.forEach(function (v) {
                        pairs.push(encodedName_1 + "=" + encodeURIComponent(v));
                    });
                }
                else if (value === undefined) {
                    return "continue";
                }
                else if (value === null) {
                    pairs.push(encodedName_1 + "=");
                }
                else {
                    pairs.push(encodedName_1 + "=" + encodeURIComponent(value));
                }
            }
        };
        for (var name_1 in query) {
            _loop_1(name_1);
        }
    }
    return pairs.join('&');
}

var index$9 = {
    get: get$1,
    parse: parse$1,
    stringify: stringify
};

/* eslint-disable */
var ascii = /^[\x00-\xff]+$/;

var decimal = /^[+-]?(?:\d+|\d+\.\d*|\.\d+)$/;

var digit = /^\d+$/;

// The most reliable method to check if a domain is valid is use the dns or
// an http verify file, like the Google Analytics.
var domain = /^[^\s:/]+\.[^\s.:/]+$/i;

// Ref: https://developer.mozilla.org/en-US/docs/Web/HTML/Element/input/email#basic_validation
var email = /^[a-zA-Z0-9.!#$%&'*+/=?^_`{|}~-]+@[a-zA-Z0-9](?:[a-zA-Z0-9-]{0,61}[a-zA-Z0-9])?(?:\.[a-zA-Z0-9](?:[a-zA-Z0-9-]{0,61}[a-zA-Z0-9])?)*$/;

// Ref: https://en.wikipedia.org/wiki/Web_colors#Hex_triplet
var hexColor = /^#([0-9a-fA-F]{3}|[0-9a-fA-F]{6})$/;

var hsl = /^(hsl|HSL)\(\s*\d+\s*,\s*\d+%\s*,\s*\d+%\s*\)$/;

var hsla = /^(hsla|HSLA)\(\s*\d+\s*,\s*\d+%\s*,\s*\d+%\s*,\s*0?\.?\d+\s*\)$/;

var integer = /^\d+$/;

var ipv4 = /^(?:25[0-5]|2[0-4][0-9]|1\d{2}|[1-9]?\d)\.(?:(?:25[0-5]|2[0-4][0-9]|1\d{2}|[1-9]?\d)\.){2}(?:25[0-5]|2[0-4][0-9]|1\d{2}|[1-9]?\d)$/;

var ipv6 = /^(([0-9a-fA-F]{1,4}:){7,7}[0-9a-fA-F]{1,4}|([0-9a-fA-F]{1,4}:){1,7}:|([0-9a-fA-F]{1,4}:){1,6}:[0-9a-fA-F]{1,4}|([0-9a-fA-F]{1,4}:){1,5}(:[0-9a-fA-F]{1,4}){1,2}|([0-9a-fA-F]{1,4}:){1,4}(:[0-9a-fA-F]{1,4}){1,3}|([0-9a-fA-F]{1,4}:){1,3}(:[0-9a-fA-F]{1,4}){1,4}|([0-9a-fA-F]{1,4}:){1,2}(:[0-9a-fA-F]{1,4}){1,5}|[0-9a-fA-F]{1,4}:((:[0-9a-fA-F]{1,4}){1,6})|:((:[0-9a-fA-F]{1,4}){1,7}|:)|fe80:(:[0-9a-fA-F]{0,4}){0,4}%[0-9a-zA-Z]{1,}|::(ffff(:0{1,4}){0,1}:){0,1}((25[0-5]|(2[0-4]|1{0,1}[0-9]){0,1}[0-9])\.){3,3}(25[0-5]|(2[0-4]|1{0,1}[0-9]){0,1}[0-9])|([0-9a-fA-F]{1,4}:){1,4}:((25[0-5]|(2[0-4]|1{0,1}[0-9]){0,1}[0-9])\.){3,3}(25[0-5]|(2[0-4]|1{0,1}[0-9]){0,1}[0-9]))$/;

var isAscii = function (string) {
    return isString(string) && ascii.test(string);
};

var isDecimal = function (string) {
    return isString(string) && decimal.test(string);
};

var isDigit$1 = function (string) {
    return isString(string) && digit.test(string);
};

var isDomain = function (string) {
    return isString(string) && domain.test(string);
};

var isEmail = function (string) {
    return isString(string) && email.test(string);
};

var isHexColor = function (string) {
    return isString(string) && hexColor.test(string);
};

var isHsl = function (string) {
    return isString(string) && hsl.test(string);
};

var isHsla = function (string) {
    return isString(string) && hsla.test(string);
};

var isInteger = function (string) {
    return isString(string) && integer.test(string);
};

var isIPv4 = function (string) {
    return isString(string) && ipv4.test(string);
};

var isIPv6 = function (string) {
    return isString(string) && ipv6.test(string);
};

var isIP = function (string) { return isIPv4(string) || isIPv6(string); };

var qq = /^1\d{4,}$/;

var isQQ = function (string) { return isString(string) && qq.test(string); };

var realNumber = /^[+-]?(?:\d+|\d+\.\d*|\.\d+)(?:[eE][+-]?\d+)?$/;

var isRealNumber = function (string) {
    return isString(string) && realNumber.test(string);
};

var rgb = /^(rgb|RGB)\(\s*\d+\s*,\s*\d+\s*,\s*\d+\s*\)$/;

var isRgb = function (string) {
    return isString(string) && rgb.test(string);
};

var rgba = /^(rgba|RGBA)\(\s*\d+\s*,\s*\d+\s*,\s*\d+\s*,\s*0?\.?\d+\s*\)$/;

var isRgba = function (string) {
    return isString(string) && rgba.test(string);
};

var index$a = {
    ascii: ascii,
    decimal: decimal,
    digit: digit,
    domain: domain,
    email: email,
    hexColor: hexColor,
    hsl: hsl,
    hsla: hsla,
    integer: integer,
    ipv4: ipv4,
    ipv6: ipv6,
    isAscii: isAscii,
    isDecimal: isDecimal,
    isDigit: isDigit$1,
    isDomain: isDomain,
    isEmail: isEmail,
    isHexColor: isHexColor,
    isHsl: isHsl,
    isHsla: isHsla,
    isInteger: isInteger,
    isIP: isIP,
    isIPv4: isIPv4,
    isIPv6: isIPv6,
    isQQ: isQQ,
    isRealNumber: isRealNumber,
    isRgb: isRgb,
    isRgba: isRgba,
    qq: qq,
    realNumber: realNumber,
    rgb: rgb,
    rgba: rgba
};

function count(str, type) {
    if (type === void 0) { type = 0; }
    if (!isString(str)) {
        throw new TypeError('the first argument must be a string');
    }
    var len = str.length;
    var num = 0;
    var fullCount = 0;
    var halfCount = 0;
    if (type === 1 || type === 2) {
        for (var i = 0; i < len; i++) {
            if (isCJK(str.charAt(i))) {
                fullCount++;
            }
            else {
                halfCount++;
            }
        }
    }
    if (type === 1) {
        num = fullCount * 2 + halfCount;
    }
    else if (type === 2) {
        num = fullCount + Math.ceil(halfCount / 2);
    }
    else {
        num = len;
    }
    return num;
}

var countLines = function (text, newline) {
    var textString = String(text);
    var newlineChar = newline || '\n';
    var index = -1;
    var count = 0;
    while ((index = textString.indexOf(newlineChar, index + 1)) > -1) {
        count++;
    }
    // Todo: use regexp to check
    var isEndWithNewline = textString.lastIndexOf(newlineChar) ===
        textString.length - newlineChar.length;
    // Todo: add a comment
    if (textString && !isEndWithNewline) {
        count++;
    }
    return count;
};

var firstChar = function (str) {
    if (!isString(str)) {
        throw new Error('The parameter 1 should be a string');
    }
    return str.charAt(0);
};

function isValidJSON(value) {
    var isValid = false;
    try {
        JSON.parse(value);
        isValid = true;
    }
    catch (err) {
        // Todo: Print a warning message into console
    }
    return isValid;
}

var lastChar = function (str) {
    if (!isString(str)) {
        throw new Error('The parameter 1 should be a string');
    }
    return str.charAt(str.length - 1);
};

function truncate(str, options) {
    if (str === void 0) { str = ''; }
    if (options === void 0) { options = {}; }
    var length = options.length, _a = options.omission, omission = _a === void 0 ? '...' : _a, _b = options.countType, countType = _b === void 0 ? 0 : _b;
    var ensureStr = String(str);
    var len = ensureStr.length;
    var countLen = length || len;
    var fullCount = countType === 1 ? 2 : 1;
    var halfCount = countType === 2 ? 0.5 : 1;
    var count = 0;
    var i = 0;
    for (; i < len; i++) {
        if (isCJK(ensureStr.charAt(i))) {
            count += fullCount;
        }
        else {
            count += halfCount;
        }
        if (count > countLen) {
            break;
        }
    }
    var truncatedStr = ensureStr.slice(0, i);
    return "" + truncatedStr + (truncatedStr.length < ensureStr.length ? omission : '');
}

var index$b = {
    countChars: count,
    countLines: countLines,
    escapeRegexp: escapeRegexp,
    firstChar: firstChar,
    isValidJSON: isValidJSON,
    lastChar: lastChar,
    truncate: truncate
};

// Ref: https://github.com/mumuy/browser
var testMap = {
    Windows: /Windows/,
    MacOS: /Macintosh/,
    Linux: /Linux|X11/,
    Android: /Android|Adr/,
    IOS: /like Mac OS X/,
    Mobile: /Mobi|iPh|480/,
    Tablet: /Tablet|Pad|Nexus 7/,
    Alipay: /AlipayClient/,
    WeChat: /MicroMessenger/,
    IPhone: /iPhone|iPod/,
    IPad: /iPad/
};
var userAgent = isBrowser ? window.navigator.userAgent : '';
function isType$1 (type) {
    return function (ua) {
        if (ua === void 0) { ua = userAgent; }
        return testMap[type].test(ua);
    };
}

var isAlipay = isType$1('Alipay');

var isAndriod = isType$1('Android');

var isIOS = isType$1('IOS');

var isIPad = isType$1('IPad');

var isIPhone = isType$1('IPhone');

var isLinux = isType$1('Linux');

var isMacOS = isType$1('MacOS');

var isMobile = isType$1('Mobile');

var isTablet = isType$1('Tablet');

var isWechat = isType$1('WeChat');

var isWindows = isType$1('Windows');

var index$c = {
    isAlipay: isAlipay,
    isAndroid: isAndriod,
    isIOS: isIOS,
    isIPad: isIPad,
    isIPhone: isIPhone,
    isLinux: isLinux,
    isMacOS: isMacOS,
    isMobile: isMobile,
    isTablet: isTablet,
    isWeChat: isWechat,
    isWindows: isWindows
};

function isAbsolute(url) {
    return isString(url) && /^https?:\/\//.test(url);
}

var reDomain = /^(https?:\/\/)?([^\s:/.]{1,63}\.){1,126}[^\s:/.]{1,63}(:\d{1,5})?\/?$/;
var reQueryOrHash = /[?#]/;
var join$1 = function () {
    var args = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        args[_i] = arguments[_i];
    }
    var len = args.length;
    var url = '';
    var sep = '';
    for (var i = 0; i < len; i++) {
        if (reQueryOrHash.test(args[i]) && sep === '/') {
            url = url.slice(0, -1);
            sep = '';
        }
        url += args[i];
        if (reDomain.test(url) && sep === '') {
            sep = '/';
        }
        if (i < len - 1) {
            url += sep;
        }
    }
    url = url.replace(/([/]{2,})/g, function (m, p1, offset) {
        if (url[offset - 1] === ':') {
            return m;
        }
        return '/';
    });
    return url;
};

var reUrl = /^(\w{1,63}):\/\/(?:(\w+)(?::(\w+))?@)?([^\s@:/?#]{1,253})(?::(\d{1,5}))?(\/[^\s?#]+)?(\?[^#]+)?(#[\w\W]+)?$/;
function parse$2(url) {
    if (!isString(url)) {
        throw new TypeError('invalid url');
    }
    var matched = url.match(reUrl);
    if (!matched) {
        throw new TypeError('invalid url');
    }
    var protocol = matched[1];
    var username = matched[2] || '';
    var password = matched[3] || '';
    var hostname = matched[4];
    var port = matched[5] || '';
    var pathname = matched[6] || '';
    var search = matched[7] || '';
    var hash = matched[8] || '';
    var host = "" + hostname + (port ? ":" + port : '');
    var origin = protocol + "://" + host;
    var query = parse$1(search);
    return {
        hash: hash,
        host: host,
        hostname: hostname,
        href: url,
        origin: origin,
        password: password,
        pathname: pathname,
        port: port,
        protocol: protocol,
        query: query,
        search: search,
        username: username
    };
}

function stringify$1(urlParts) {
    if (!isObject(urlParts)) {
        throw new TypeError('invalid url parts object');
    }
    var protocol = urlParts.protocol, username = urlParts.username, password = urlParts.password, host = urlParts.host, hostname = urlParts.hostname, _a = urlParts.port, port = _a === void 0 ? '' : _a, _b = urlParts.pathname, pathname = _b === void 0 ? '' : _b, _c = urlParts.search, search = _c === void 0 ? '' : _c, _d = urlParts.query, query = _d === void 0 ? {} : _d, _e = urlParts.hash, hash = _e === void 0 ? '' : _e;
    if (!protocol) {
        throw new TypeError('protocol missing');
    }
    if (!host && !hostname) {
        throw new TypeError('host or hostname missing');
    }
    var url = '';
    url += protocol + "://";
    if (username) {
        url += username;
        if (password) {
            url += ":" + password;
        }
        url += '@';
    }
    if (host) {
        url += host;
    }
    else {
        url += hostname;
        if (port) {
            url += ":" + port;
        }
    }
    url += pathname;
    if (search) {
        url += search;
    }
    else {
        /* istanbul ignore else */
        if (query) {
            var search_1 = stringify(query);
            if (search_1) {
                url += "?" + search_1;
            }
        }
    }
    url += hash;
    return url;
}

var index$d = {
    isAbsolute: isAbsolute,
    join: join$1,
    parse: parse$2,
    stringify: stringify$1
};

function ajax(url, options) {
    if (options === void 0) { options = {}; }
    return new Promise(function (resolve, reject) {
        var _a = options.async, async = _a === void 0 ? true : _a, _b = options.beforeSend, beforeSend = _b === void 0 ? function () { } : _b, _c = options.data, data = _c === void 0 ? null : _c, _d = options.headers, headers = _d === void 0 ? {} : _d, _e = options.method, method = _e === void 0 ? 'get' : _e, _f = options.responseType, responseType = _f === void 0 ? 'json' : _f, _g = options.timeout, timeout = _g === void 0 ? 0 : _g;
        var xhr = new XMLHttpRequest();
        xhr.open(method, url, async);
        var timerId;
        if (timeout) {
            timerId = window.setTimeout(function () {
                reject(new Error("the xhr request timeout " + timeout + "ms"));
            }, timeout);
        }
        xhr.onerror = function () {
            reject(new Error('unknown error'));
        };
        xhr.onreadystatechange = function () {
            if (xhr.readyState === 4) {
                if (timeout && timerId !== undefined)
                    clearTimeout(timerId);
                if (xhr.status >= 200 && xhr.status < 300) {
                    resolve(xhr);
                }
                else {
                    var err = new Error('the request is error');
                    reject(err);
                }
            }
        };
        var sendData = null;
        if (isObject(data)) {
            try {
                sendData = JSON.stringify(data);
                headers['content-type'] = 'application/json';
            }
            catch (err) {
                reject(err);
            }
        }
        else if (isString(data)) {
            headers['content-type'] = 'application/x-www-form-urlencoded';
        }
        for (var key in headers) {
            xhr.setRequestHeader(key, headers[key]);
        }
        if (timeout) {
            xhr.timeout = timeout;
        }
        xhr.responseType = responseType;
        if (beforeSend(xhr) !== false) {
            xhr.send(sendData);
        }
    });
}

function classNames() {
    var args = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        args[_i] = arguments[_i];
    }
    var cls = '';
    for (var _a = 0, args_1 = args; _a < args_1.length; _a++) {
        var arg = args_1[_a];
        if (isString(arg)) {
            cls += " " + arg;
        }
        else if (isObject(arg)) {
            var keys = Object.keys(arg);
            for (var _b = 0, keys_1 = keys; _b < keys_1.length; _b++) {
                var key = keys_1[_b];
                if (arg[key]) {
                    cls += " " + key;
                }
            }
        }
        else if (isArray(arg)) {
            cls += " " + classNames.apply(void 0, arg);
        }
    }
    return cls.slice(1);
}

function css(el, prop) {
    var elem;
    if (isString(el)) {
        elem = document.querySelector(el);
        if (!elem) {
            throw new Error('the element is not exist');
        }
    }
    else if (isElement(el)) {
        elem = el;
    }
    else {
        throw new Error('the 1st argument must be a css selector or an element');
    }
    if (isObject(prop)) {
        var cssText_1 = elem.style.cssText;
        Object.keys(prop).forEach(function (name) {
            cssText_1 += name + ": " + prop[name] + ";";
        });
        elem.style.cssText = cssText_1;
    }
    else {
        var ret = window.getComputedStyle(elem);
        if (isString(prop)) {
            // @ts-ignore
            ret = ret[prop];
        }
        return ret;
    }
}

function delay(duration) {
    if (duration === void 0) { duration = 0; }
    return new Promise(function (resolve) { return setTimeout(function () { return resolve(); }, duration); });
}

/*
 * Modified FileSaver.js
 * A saveAs() FileSaver implementation.
 *
 * By Eli Grey, http://eligrey.com
 *
 * License : https://github.com/eligrey/FileSaver.js/blob/master/LICENSE.md (MIT)
 * source  : http://purl.eligrey.com/github/FileSaver.js
 */
// TODO: add tests
var isMacOSWebView = function () {
    return /Macintosh/.test(navigator.userAgent) &&
        /AppleWebKit/.test(navigator.userAgent) &&
        !/Safari/.test(navigator.userAgent);
};
function bom(file) {
    if (/^\s*(?:text\/[^;\s/]*|application\/xml|[^;\s/]*\/[^;\s/]*\+xml)\s*;.*charset\s*=\s*utf-8/i.test(file.type)) {
        return new Blob([String.fromCharCode(0xfeff), file], { type: file.type });
    }
    return file;
}
function corsEnabled(url) {
    var xhr = new XMLHttpRequest();
    xhr.open('HEAD', url, false);
    try {
        xhr.send();
        // eslint-disable-next-line
    }
    catch (e) { }
    return xhr.status >= 200 && xhr.status <= 299;
}
function downloadUrl(url, fileName) {
    var xhr = new XMLHttpRequest();
    xhr.open('GET', url);
    xhr.responseType = 'blob';
    xhr.onload = function () {
        download(xhr.response, fileName);
    };
    xhr.onerror = function () {
        console.error('could not download file');
    };
    xhr.send();
}
function click(node) {
    try {
        node.dispatchEvent(new MouseEvent('click'));
    }
    catch (e) {
        var evt = document.createEvent('MouseEvents');
        evt.initMouseEvent('click', true, true, window, 0, 0, 0, 80, 20, false, false, false, false, 0, null);
        node.dispatchEvent(evt);
    }
}
var download;
/* istanbul ignore else */
if (!isBrowser) {
    download = function download() {
        throw new Error('download is not support in pure Node.js');
    };
}
else if ('download' in HTMLAnchorElement.prototype && !isMacOSWebView()) {
    download = function download(file, name) {
        var a = document.createElement('a');
        var fileName = name || file.name || 'download';
        a.download = fileName;
        a.rel = 'noopener';
        if (typeof file === 'string') {
            a.href = file;
            if (a.origin !== location.origin) {
                if (corsEnabled(a.href)) {
                    downloadUrl(file, fileName);
                }
                else {
                    a.target = '_blank';
                    click(a);
                }
            }
            else {
                click(a);
            }
        }
        else {
            var URL_1 = window.URL || window.webkitURL;
            a.href = URL_1.createObjectURL(file);
            setTimeout(function () { return URL_1.revokeObjectURL(a.href); }, 4e4); // 40s
            setTimeout(function () { return click(a); }, 0);
        }
    };
}
else {
    if ('msSaveOrOpenBlob' in navigator) {
        download = function download(file, name) {
            var fileName = name || file.name || 'download';
            if (typeof file === 'string') {
                if (corsEnabled(file)) {
                    downloadUrl(file, fileName);
                }
                else {
                    var a_1 = document.createElement('a');
                    a_1.href = file;
                    a_1.target = '_blank';
                    setTimeout(function () { return click(a_1); });
                }
            }
            else {
                navigator.msSaveOrOpenBlob(bom(file), fileName);
            }
        };
    }
    else {
        download = function download(file, name) {
            var popup = window.open('', '_blank');
            if (popup) {
                popup.document.title = 'downloading...';
                popup.document.body.innerText = 'downloading...';
            }
            var fileName = name || file.name || 'download';
            if (typeof file === 'string') {
                downloadUrl(file, fileName);
                return;
            }
            var force = file.type === 'application/octet-stream';
            var isSafari = /constructor/i.test(String(window.HTMLElement)) ||
                String(window.safari || '');
            var isChromeIOS = /CriOS\/[\d]+/.test(navigator.userAgent);
            if ((isChromeIOS || (force && isSafari) || isMacOSWebView()) &&
                typeof FileReader !== 'undefined') {
                // Safari doesn't allow downloading of blob URLs
                var reader_1 = new FileReader();
                reader_1.onloadend = function () {
                    var url = reader_1.result || '';
                    url = isChromeIOS
                        ? url
                        : url.replace(/^data:[^;]*;/, 'data:attachment/file;');
                    if (popup) {
                        popup.location.href = url;
                    }
                    else {
                        window.location.href = url;
                    }
                    popup = null;
                };
                reader_1.readAsDataURL(file);
            }
            else {
                var URL_2 = window.URL || window.webkitURL;
                var url_1 = URL_2.createObjectURL(file);
                if (popup)
                    popup.location.href = url_1;
                else
                    location.href = url_1;
                popup = null;
                setTimeout(function () {
                    URL_2.revokeObjectURL(url_1);
                }, 4e4); // 40s
            }
        };
    }
}
var downloadAlias = download;

function jsonp(url, options) {
    if (options === void 0) { options = {}; }
    return new Promise(function (resolve, reject) {
        var _a = options.callback, callback = _a === void 0 ? '_callback' : _a, _b = options.responseType, responseType = _b === void 0 ? 'json' : _b, _c = options.timeout, timeout = _c === void 0 ? 0 : _c;
        var callbackName = "_callback" + Date.now();
        var hasQuestion = url.indexOf('?') > -1;
        var src = url;
        if (hasQuestion) {
            src += '&';
        }
        else {
            src += '?';
        }
        src += encodeURIComponent(callback) + "=" + encodeURIComponent(callbackName);
        var timerId;
        if (timeout) {
            // use `window.setTimeout` instead of `setTimeout` directly because type check
            timerId = window.setTimeout(function () {
                reject(new Error("the request is timeout " + timeout + "ms"));
            }, timeout);
        }
        // @ts-ignore
        window[callbackName] = function (text) {
            if (timerId)
                clearTimeout(timerId);
            if (responseType === 'json') {
                try {
                    resolve(JSON.parse(text));
                }
                catch (err) {
                    reject(err);
                }
            }
            else {
                resolve(text);
            }
        };
        insertScript(src, {
            onerror: function () {
                reject(new Error('load error'));
            }
        });
    });
}

function bytesToWords(bytes) {
    var words = [];
    for (var i = 0, b = 0; i < bytes.length; i++, b += 8) {
        words[b >>> 5] |= bytes[i] << (24 - (b % 32));
    }
    return words;
}

function wordsToBytes(words) {
    var bytes = [];
    for (var i = 0; i < words.length * 32; i += 8) {
        bytes.push((words[i >>> 5] >>> (24 - (i % 32))) & 0xff);
    }
    return bytes;
}

function swapEndian(arr) {
    for (var i = 0; i < arr.length; i++) {
        var n = arr[i];
        arr[i] =
            (((n << 8) | (n >>> 24)) & 0x00ff00ff) |
                (((n << 24) | (n >>> 8)) & 0xff00ff00);
    }
    return arr;
}

var isUint8Array$1 = isType('Uint8Array');
function F(a, b, c, d, x, s, t) {
    var n = a + ((b & c) | (~b & d)) + (x >>> 0) + t;
    return ((n << s) | (n >>> (32 - s))) + b;
}
function G(a, b, c, d, x, s, t) {
    var n = a + ((b & d) | (c & ~d)) + (x >>> 0) + t;
    return ((n << s) | (n >>> (32 - s))) + b;
}
function H(a, b, c, d, x, s, t) {
    var n = a + (b ^ c ^ d) + (x >>> 0) + t;
    return ((n << s) | (n >>> (32 - s))) + b;
}
function I(a, b, c, d, x, s, t) {
    var n = a + (c ^ (b | ~d)) + (x >>> 0) + t;
    return ((n << s) | (n >>> (32 - s))) + b;
}
function md5(input) {
    var bytes = [];
    if (isString(input)) {
        bytes = utf8ToBytes(input);
    }
    else if (isArray(input) || isUint8Array$1(input)) {
        bytes = input;
    }
    else {
        throw TypeError('the input parameter must be one of String, Byte Array, Uint8Array');
    }
    var m = bytesToWords(bytes);
    var len = bytes.length * 8;
    var a = 1732584193;
    var b = -271733879;
    var c = -1732584194;
    var d = 271733878;
    // Swap endian
    for (var i = 0; i < m.length; i++) {
        m[i] =
            (((m[i] << 8) | (m[i] >>> 24)) & 0x00ff00ff) |
                (((m[i] << 24) | (m[i] >>> 8)) & 0xff00ff00);
    }
    // Padding
    m[len >>> 5] |= 0x80 << len % 32;
    m[(((len + 64) >>> 9) << 4) + 14] = len;
    for (var i = 0; i < m.length; i += 16) {
        var aa = a;
        var bb = b;
        var cc = c;
        var dd = d;
        a = F(a, b, c, d, m[i + 0], 7, -680876936);
        d = F(d, a, b, c, m[i + 1], 12, -389564586);
        c = F(c, d, a, b, m[i + 2], 17, 606105819);
        b = F(b, c, d, a, m[i + 3], 22, -1044525330);
        a = F(a, b, c, d, m[i + 4], 7, -176418897);
        d = F(d, a, b, c, m[i + 5], 12, 1200080426);
        c = F(c, d, a, b, m[i + 6], 17, -1473231341);
        b = F(b, c, d, a, m[i + 7], 22, -45705983);
        a = F(a, b, c, d, m[i + 8], 7, 1770035416);
        d = F(d, a, b, c, m[i + 9], 12, -1958414417);
        c = F(c, d, a, b, m[i + 10], 17, -42063);
        b = F(b, c, d, a, m[i + 11], 22, -1990404162);
        a = F(a, b, c, d, m[i + 12], 7, 1804603682);
        d = F(d, a, b, c, m[i + 13], 12, -40341101);
        c = F(c, d, a, b, m[i + 14], 17, -1502002290);
        b = F(b, c, d, a, m[i + 15], 22, 1236535329);
        a = G(a, b, c, d, m[i + 1], 5, -165796510);
        d = G(d, a, b, c, m[i + 6], 9, -1069501632);
        c = G(c, d, a, b, m[i + 11], 14, 643717713);
        b = G(b, c, d, a, m[i + 0], 20, -373897302);
        a = G(a, b, c, d, m[i + 5], 5, -701558691);
        d = G(d, a, b, c, m[i + 10], 9, 38016083);
        c = G(c, d, a, b, m[i + 15], 14, -660478335);
        b = G(b, c, d, a, m[i + 4], 20, -405537848);
        a = G(a, b, c, d, m[i + 9], 5, 568446438);
        d = G(d, a, b, c, m[i + 14], 9, -1019803690);
        c = G(c, d, a, b, m[i + 3], 14, -187363961);
        b = G(b, c, d, a, m[i + 8], 20, 1163531501);
        a = G(a, b, c, d, m[i + 13], 5, -1444681467);
        d = G(d, a, b, c, m[i + 2], 9, -51403784);
        c = G(c, d, a, b, m[i + 7], 14, 1735328473);
        b = G(b, c, d, a, m[i + 12], 20, -1926607734);
        a = H(a, b, c, d, m[i + 5], 4, -378558);
        d = H(d, a, b, c, m[i + 8], 11, -2022574463);
        c = H(c, d, a, b, m[i + 11], 16, 1839030562);
        b = H(b, c, d, a, m[i + 14], 23, -35309556);
        a = H(a, b, c, d, m[i + 1], 4, -1530992060);
        d = H(d, a, b, c, m[i + 4], 11, 1272893353);
        c = H(c, d, a, b, m[i + 7], 16, -155497632);
        b = H(b, c, d, a, m[i + 10], 23, -1094730640);
        a = H(a, b, c, d, m[i + 13], 4, 681279174);
        d = H(d, a, b, c, m[i + 0], 11, -358537222);
        c = H(c, d, a, b, m[i + 3], 16, -722521979);
        b = H(b, c, d, a, m[i + 6], 23, 76029189);
        a = H(a, b, c, d, m[i + 9], 4, -640364487);
        d = H(d, a, b, c, m[i + 12], 11, -421815835);
        c = H(c, d, a, b, m[i + 15], 16, 530742520);
        b = H(b, c, d, a, m[i + 2], 23, -995338651);
        a = I(a, b, c, d, m[i + 0], 6, -198630844);
        d = I(d, a, b, c, m[i + 7], 10, 1126891415);
        c = I(c, d, a, b, m[i + 14], 15, -1416354905);
        b = I(b, c, d, a, m[i + 5], 21, -57434055);
        a = I(a, b, c, d, m[i + 12], 6, 1700485571);
        d = I(d, a, b, c, m[i + 3], 10, -1894986606);
        c = I(c, d, a, b, m[i + 10], 15, -1051523);
        b = I(b, c, d, a, m[i + 1], 21, -2054922799);
        a = I(a, b, c, d, m[i + 8], 6, 1873313359);
        d = I(d, a, b, c, m[i + 15], 10, -30611744);
        c = I(c, d, a, b, m[i + 6], 15, -1560198380);
        b = I(b, c, d, a, m[i + 13], 21, 1309151649);
        a = I(a, b, c, d, m[i + 4], 6, -145523070);
        d = I(d, a, b, c, m[i + 11], 10, -1120210379);
        c = I(c, d, a, b, m[i + 2], 15, 718787259);
        b = I(b, c, d, a, m[i + 9], 21, -343485551);
        a = (a + aa) >>> 0;
        b = (b + bb) >>> 0;
        c = (c + cc) >>> 0;
        d = (d + dd) >>> 0;
    }
    return bytesToHex(wordsToBytes(swapEndian([a, b, c, d])));
}

function noop() { }

function numberFormat(number, digits, dot, sep) {
    if (digits === void 0) { digits = -1; }
    if (dot === void 0) { dot = '.'; }
    if (sep === void 0) { sep = ','; }
    if (!isNumber(number)) {
        return '';
    }
    var tenExp = Math.pow(10, digits);
    var result = number >= 0 ? '' : '-';
    var decPartStr;
    if (digits > -1) {
        number = Math.round(number * tenExp) / tenExp;
    }
    var abs = Math.abs(number);
    var intPart = Math.floor(abs);
    var decPart = Math.round(abs * tenExp - intPart * tenExp) / tenExp;
    var intPartStr = intPart.toString();
    if (decPart === 0) {
        decPartStr = '';
    }
    else {
        decPartStr = decPart.toString().slice(2);
    }
    if (digits > -1) {
        for (var i = 0; i < digits; i++) {
            if (!decPartStr[i]) {
                decPartStr += '0';
            }
        }
        decPartStr = decPartStr.slice(0, digits);
    }
    var intLen = intPartStr.length;
    var start = intLen % 3;
    for (var i = 0; i < intLen; i++) {
        result += intPartStr[i];
        if ((i + 1 - start) % 3 === 0 && i !== intLen - 1) {
            result += sep;
        }
    }
    if (decPartStr) {
        result += dot;
        result += decPartStr;
    }
    return result;
}

function randomNumbers(start, end, length) {
    if (end - start + 1 < length) {
        throw new RangeError('the result of end - start + 1 should be greater than length');
    }
    var obj = Object.create(null);
    var nums = [];
    var count = 0;
    var num;
    while (count < length) {
        num = Math.floor(Math.random() * (end - start + 1)) + start;
        if (num in obj)
            continue;
        nums.push(num);
        count++;
        obj[num] = true;
    }
    return nums;
}

/**
 * A simple function can be used to generate a random string with the specified length.
 * And,the range of the characters is configurable.
 *
 * Default:
 *  len: 32
 *  chars: '0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghiklmnopqrstuvwxyz'
 *
 * Note:
 *  The method charAt will convert its first parameter to an integer by floor.
 *  See the specification: http://www.ecma-international.org/ecma-262/5.1/#sec-15.5.4.4
 *
 * Performance:
 *  http://jsperf.com/random-string-for-loop-vs-replace
 */
function randomString(len, chars) {
    var actualLen = len || 32;
    var actualChars = chars || '0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghiklmnopqrstuvwxyz';
    var str = '';
    for (var i = 0; i < actualLen; i++) {
        str += actualChars.charAt(Math.random() * actualChars.length);
    }
    return str;
}

/**
 * Generate a uuid from pseudo-random numbers
 *
 * Ref: https://tools.ietf.org/html/rfc4122
 */
var hexDigits = '0123456789abcdef';
var maxUint32 = Math.pow(2, 32) - 1;
var random = Math.random;
if (isBrowser) {
    var getRandomValues_1;
    /* istanbul ignore else */
    if (window.crypto && typeof window.crypto.getRandomValues === 'function') {
        getRandomValues_1 = function (array) {
            return window.crypto.getRandomValues(array);
        };
    }
    else {
        // @ts-ignore
        if (window.msCrypto &&
            typeof window.msCrypto.getRandomValues === 'function') {
            // @ts-ignore
            getRandomValues_1 = function (array) {
                return window.msCrypto.getRandomValues(array);
            };
        }
    }
    /* istanbul ignore else */
    // @ts-ignore
    if (typeof window.Uint32Array === 'function' && getRandomValues_1) {
        random = function () { return getRandomValues_1(new Uint32Array(1))[0] / maxUint32; };
    }
}
function uuid() {
    var u = [];
    for (var i = 0; i < 36; i++) {
        u[i] = hexDigits.charAt(random() * 16);
    }
    u[8] = u[13] = u[18] = u[23] = '-';
    u[14] = '4';
    u[19] = ((Number(u[19]) & 3) | 8).toString(16);
    return u.join('');
}

var micell;
(function (micell) {
    micell.ajax = ajax;
    micell.classNames = classNames;
    micell.css = css;
    micell.delay = delay;
    micell.download = downloadAlias;
    micell.jsonp = jsonp;
    micell.md5 = md5;
    micell.noop = noop;
    micell.numberFormat = numberFormat;
    micell.raf = exposeRaf;
    micell.randomNumbers = randomNumbers;
    micell.randomString = randomString;
    micell.uuid = uuid;
    (function (base64) {
        base64.decode = index.decode;
        base64.decodeFile = index.decodeFile;
        base64.encode = index.encode;
        base64.encodeFile = index.encodeFile;
    })(micell.base64 || (micell.base64 = {}));
    (function (char) {
        char.isAlphabetic = index$1.isAlphabetic;
        char.isAlphanumeric = index$1.isAlphanumeric;
        char.isASCII = index$1.isASCII;
        char.isCJK = index$1.isCJK;
        char.isControl = index$1.isControl;
        char.isDigit = index$1.isDigit;
        char.isGraph = index$1.isGraph;
        char.isHexadecimal = index$1.isHexadecimal;
        char.isLower = index$1.isLower;
        char.isPrint = index$1.isPrint;
        char.isPunctuation = index$1.isPunctuation;
        char.isSpace = index$1.isSpace;
        char.isUpper = index$1.isUpper;
    })(micell.char || (micell.char = {}));
    (function (cookie) {
        cookie.get = index$2.get;
        cookie.getAll = index$2.getAll;
        cookie.getJson = index$2.getJson;
        cookie.getRaw = index$2.getRaw;
        cookie.isEnabled = index$2.isEnabled;
        cookie.remove = index$2.remove;
        cookie.set = index$2.set;
        cookie.setJson = index$2.setJson;
        cookie.setRaw = index$2.setRaw;
    })(micell.cookie || (micell.cookie = {}));
    (function (date) {
        date.diff = index$3.diff;
        date.format = index$3.format;
        date.parse = index$3.parse;
        date.timeAgo = index$3.timeAgo;
    })(micell.date || (micell.date = {}));
    (function (dom) {
        dom.clientX = index$4.clientX;
        dom.clientY = index$4.clientY;
        dom.createEvent = index$4.createEvent;
        dom.domReady = index$4.domReady;
        dom.insertScript = index$4.insertScript;
        dom.isElement = index$4.isElement;
        dom.isInViewport = index$4.isInViewport;
        dom.isWindow = index$4.isWindow;
        dom.pageHeight = index$4.pageHeight;
        dom.pageWidth = index$4.pageWidth;
        dom.pageX = index$4.pageX;
        dom.pageY = index$4.pageY;
        dom.scrollTo = index$4.scrollTo;
        dom.scrollX = index$4.scrollX;
        dom.scrollY = index$4.scrollY;
        dom.viewport = index$4.viewport;
    })(micell.dom || (micell.dom = {}));
    (function (easing) {
        easing.cubicBezier = index$5.cubicBezier;
        easing.ease = index$5.ease;
        easing.easeIn = index$5.easeIn;
        easing.easeInOut = index$5.easeInOut;
        easing.easeOut = index$5.easeOut;
        easing.linear = index$5.linear;
    })(micell.easing || (micell.easing = {}));
    (function (emitter) {
        emitter.create = index$6.create;
        emitter.emit = index$6.emit;
        emitter.Emitter = index$6.Emitter;
        emitter.getListeners = index$6.getListeners;
        emitter.once = index$6.once;
        emitter.on = index$6.on;
        emitter.off = index$6.off;
    })(micell.emitter || (micell.emitter = {}));
    (function (lang) {
        lang.isArray = index$7.isArray;
        lang.isBoolean = index$7.isBoolean;
        lang.isDate = index$7.isDate;
        lang.isError = index$7.isError;
        lang.isFunction = index$7.isFunction;
        lang.isNil = index$7.isNil;
        lang.isNull = index$7.isNull;
        lang.isNumber = index$7.isNumber;
        lang.isObject = index$7.isObject;
        lang.isRegExp = index$7.isRegExp;
        lang.isString = index$7.isString;
        lang.isUndefined = index$7.isUndefined;
    })(micell.lang || (micell.lang = {}));
    (function (path) {
        path.basename = index$8.basename;
        path.dirname = index$8.dirname;
        path.extname = index$8.extname;
        path.join = index$8.join;
    })(micell.path || (micell.path = {}));
    (function (qs) {
        qs.get = index$9.get;
        qs.parse = index$9.parse;
        qs.stringify = index$9.stringify;
    })(micell.qs || (micell.qs = {}));
    (function (regex) {
        regex.ascii = index$a.ascii;
        regex.decimal = index$a.decimal;
        regex.digit = index$a.digit;
        regex.domain = index$a.domain;
        regex.email = index$a.email;
        regex.hexColor = index$a.hexColor;
        regex.hsl = index$a.hsl;
        regex.hsla = index$a.hsla;
        regex.integer = index$a.integer;
        regex.ipv4 = index$a.ipv4;
        regex.ipv6 = index$a.ipv6;
        regex.isAscii = index$a.isAscii;
        regex.isDecimal = index$a.isDecimal;
        regex.isDigit = index$a.isDigit;
        regex.isDomain = index$a.isDomain;
        regex.isEmail = index$a.isEmail;
        regex.isHexColor = index$a.isHexColor;
        regex.isHsl = index$a.isHsl;
        regex.isHsla = index$a.isHsla;
        regex.isIP = index$a.isIP;
        regex.isIPv4 = index$a.isIPv4;
        regex.isIPv6 = index$a.isIPv6;
        regex.isInteger = index$a.isInteger;
        regex.isQQ = index$a.isQQ;
        regex.isRealNumber = index$a.isRealNumber;
        regex.isRgb = index$a.isRgb;
        regex.isRgba = index$a.isRgba;
        regex.qq = index$a.qq;
        regex.realNumber = index$a.realNumber;
        regex.rgb = index$a.rgb;
        regex.rgba = index$a.rgba;
    })(micell.regex || (micell.regex = {}));
    (function (string) {
        string.countChars = index$b.countChars;
        string.countLines = index$b.countLines;
        string.escapeRegexp = index$b.escapeRegexp;
        string.firstChar = index$b.firstChar;
        string.isValidJSON = index$b.isValidJSON;
        string.lastChar = index$b.lastChar;
        string.truncate = index$b.truncate;
    })(micell.string || (micell.string = {}));
    (function (url) {
        url.isAbsolute = index$d.isAbsolute;
        url.join = index$d.join;
        url.parse = index$d.parse;
        url.stringify = index$d.stringify;
    })(micell.url || (micell.url = {}));
    (function (ua) {
        ua.isAndroid = index$c.isAndroid;
        ua.isIOS = index$c.isIOS;
        ua.isIPad = index$c.isIPad;
        ua.isIPhone = index$c.isIPhone;
        ua.isLinux = index$c.isLinux;
        ua.isMacOS = index$c.isMacOS;
        ua.isMobile = index$c.isMobile;
        ua.isTablet = index$c.isTablet;
        ua.isWeChat = index$c.isWeChat;
        ua.isWindows = index$c.isWindows;
    })(micell.ua || (micell.ua = {}));
})(micell || (micell = {}));
var micell$1 = micell;

export default micell$1;
//# sourceMappingURL=micell.esm.js.map
