"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;

var _isBrowser = _interopRequireDefault(require("./_internal/isBrowser"));

/*
 * Modified FileSaver.js
 * A saveAs() FileSaver implementation.
 *
 * By Eli Grey, http://eligrey.com
 *
 * License : https://github.com/eligrey/FileSaver.js/blob/master/LICENSE.md (MIT)
 * source  : http://purl.eligrey.com/github/FileSaver.js
 */
// TODO: add tests
var isMacOSWebView = function isMacOSWebView() {
  return /Macintosh/.test(navigator.userAgent) && /AppleWebKit/.test(navigator.userAgent) && !/Safari/.test(navigator.userAgent);
};

function bom(file) {
  if (/^\s*(?:text\/[^;\s/]*|application\/xml|[^;\s/]*\/[^;\s/]*\+xml)\s*;.*charset\s*=\s*utf-8/i.test(file.type)) {
    return new Blob([String.fromCharCode(0xfeff), file], {
      type: file.type
    });
  }

  return file;
}

function corsEnabled(url) {
  var xhr = new XMLHttpRequest();
  xhr.open('HEAD', url, false);

  try {
    xhr.send(); // eslint-disable-next-line
  } catch (e) {}

  return xhr.status >= 200 && xhr.status <= 299;
}

function downloadUrl(url, fileName) {
  var xhr = new XMLHttpRequest();
  xhr.open('GET', url);
  xhr.responseType = 'blob';

  xhr.onload = function () {
    download(xhr.response, fileName);
  };

  xhr.onerror = function () {
    console.error('could not download file');
  };

  xhr.send();
}

function click(node) {
  try {
    node.dispatchEvent(new MouseEvent('click'));
  } catch (e) {
    var evt = document.createEvent('MouseEvents');
    evt.initMouseEvent('click', true, true, window, 0, 0, 0, 80, 20, false, false, false, false, 0, null);
    node.dispatchEvent(evt);
  }
}

var download;
/* istanbul ignore else */

if (!_isBrowser["default"]) {
  download = function download() {
    throw new Error('download is not support in pure Node.js');
  };
} else if ('download' in HTMLAnchorElement.prototype && !isMacOSWebView()) {
  download = function download(file, name) {
    var a = document.createElement('a');
    var fileName = name || file.name || 'download';
    a.download = fileName;
    a.rel = 'noopener';

    if (typeof file === 'string') {
      a.href = file;

      if (a.origin !== location.origin) {
        if (corsEnabled(a.href)) {
          downloadUrl(file, fileName);
        } else {
          a.target = '_blank';
          click(a);
        }
      } else {
        click(a);
      }
    } else {
      var URL = window.URL || window.webkitURL;
      a.href = URL.createObjectURL(file);
      setTimeout(function () {
        return URL.revokeObjectURL(a.href);
      }, 4e4); // 40s

      setTimeout(function () {
        return click(a);
      }, 0);
    }
  };
} else {
  if ('msSaveOrOpenBlob' in navigator) {
    download = function download(file, name) {
      var fileName = name || file.name || 'download';

      if (typeof file === 'string') {
        if (corsEnabled(file)) {
          downloadUrl(file, fileName);
        } else {
          var a = document.createElement('a');
          a.href = file;
          a.target = '_blank';
          setTimeout(function () {
            return click(a);
          });
        }
      } else {
        navigator.msSaveOrOpenBlob(bom(file), fileName);
      }
    };
  } else {
    download = function download(file, name) {
      var popup = window.open('', '_blank');

      if (popup) {
        popup.document.title = 'downloading...';
        popup.document.body.innerText = 'downloading...';
      }

      var fileName = name || file.name || 'download';

      if (typeof file === 'string') {
        downloadUrl(file, fileName);
        return;
      }

      var force = file.type === 'application/octet-stream';
      var isSafari = /constructor/i.test(String(window.HTMLElement)) || String(window.safari || '');
      var isChromeIOS = /CriOS\/[\d]+/.test(navigator.userAgent);

      if ((isChromeIOS || force && isSafari || isMacOSWebView()) && typeof FileReader !== 'undefined') {
        // Safari doesn't allow downloading of blob URLs
        var reader = new FileReader();

        reader.onloadend = function () {
          var url = reader.result || '';
          url = isChromeIOS ? url : url.replace(/^data:[^;]*;/, 'data:attachment/file;');

          if (popup) {
            popup.location.href = url;
          } else {
            window.location.href = url;
          }

          popup = null;
        };

        reader.readAsDataURL(file);
      } else {
        var URL = window.URL || window.webkitURL;
        var url = URL.createObjectURL(file);
        if (popup) popup.location.href = url;else location.href = url;
        popup = null;
        setTimeout(function () {
          URL.revokeObjectURL(url);
        }, 4e4); // 40s
      }
    };
  }
}

var _default = download;
exports["default"] = _default;
module.exports = exports.default;