"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = md5;

var _isType = _interopRequireDefault(require("./_internal/isType"));

var _utf8ToBytes = _interopRequireDefault(require("./_internal/utf8ToBytes"));

var _bytesToWords = _interopRequireDefault(require("./_internal/bytesToWords"));

var _wordsToBytes = _interopRequireDefault(require("./_internal/wordsToBytes"));

var _bytesToHex = _interopRequireDefault(require("./_internal/bytesToHex"));

var _swapEndian = _interopRequireDefault(require("./_internal/swapEndian"));

var _isString = _interopRequireDefault(require("./lang/isString"));

var _isArray = _interopRequireDefault(require("./lang/isArray"));

var isUint8Array = (0, _isType["default"])('Uint8Array');

function F(a, b, c, d, x, s, t) {
  var n = a + (b & c | ~b & d) + (x >>> 0) + t;
  return (n << s | n >>> 32 - s) + b;
}

function G(a, b, c, d, x, s, t) {
  var n = a + (b & d | c & ~d) + (x >>> 0) + t;
  return (n << s | n >>> 32 - s) + b;
}

function H(a, b, c, d, x, s, t) {
  var n = a + (b ^ c ^ d) + (x >>> 0) + t;
  return (n << s | n >>> 32 - s) + b;
}

function I(a, b, c, d, x, s, t) {
  var n = a + (c ^ (b | ~d)) + (x >>> 0) + t;
  return (n << s | n >>> 32 - s) + b;
}

function md5(input) {
  var bytes = [];

  if ((0, _isString["default"])(input)) {
    bytes = (0, _utf8ToBytes["default"])(input);
  } else if ((0, _isArray["default"])(input) || isUint8Array(input)) {
    bytes = input;
  } else {
    throw TypeError('the input parameter must be one of String, Byte Array, Uint8Array');
  }

  var m = (0, _bytesToWords["default"])(bytes);
  var len = bytes.length * 8;
  var a = 1732584193;
  var b = -271733879;
  var c = -1732584194;
  var d = 271733878; // Swap endian

  for (var i = 0; i < m.length; i++) {
    m[i] = (m[i] << 8 | m[i] >>> 24) & 0x00ff00ff | (m[i] << 24 | m[i] >>> 8) & 0xff00ff00;
  } // Padding


  m[len >>> 5] |= 0x80 << len % 32;
  m[(len + 64 >>> 9 << 4) + 14] = len;

  for (var _i = 0; _i < m.length; _i += 16) {
    var aa = a;
    var bb = b;
    var cc = c;
    var dd = d;
    a = F(a, b, c, d, m[_i + 0], 7, -680876936);
    d = F(d, a, b, c, m[_i + 1], 12, -389564586);
    c = F(c, d, a, b, m[_i + 2], 17, 606105819);
    b = F(b, c, d, a, m[_i + 3], 22, -1044525330);
    a = F(a, b, c, d, m[_i + 4], 7, -176418897);
    d = F(d, a, b, c, m[_i + 5], 12, 1200080426);
    c = F(c, d, a, b, m[_i + 6], 17, -1473231341);
    b = F(b, c, d, a, m[_i + 7], 22, -45705983);
    a = F(a, b, c, d, m[_i + 8], 7, 1770035416);
    d = F(d, a, b, c, m[_i + 9], 12, -1958414417);
    c = F(c, d, a, b, m[_i + 10], 17, -42063);
    b = F(b, c, d, a, m[_i + 11], 22, -1990404162);
    a = F(a, b, c, d, m[_i + 12], 7, 1804603682);
    d = F(d, a, b, c, m[_i + 13], 12, -40341101);
    c = F(c, d, a, b, m[_i + 14], 17, -1502002290);
    b = F(b, c, d, a, m[_i + 15], 22, 1236535329);
    a = G(a, b, c, d, m[_i + 1], 5, -165796510);
    d = G(d, a, b, c, m[_i + 6], 9, -1069501632);
    c = G(c, d, a, b, m[_i + 11], 14, 643717713);
    b = G(b, c, d, a, m[_i + 0], 20, -373897302);
    a = G(a, b, c, d, m[_i + 5], 5, -701558691);
    d = G(d, a, b, c, m[_i + 10], 9, 38016083);
    c = G(c, d, a, b, m[_i + 15], 14, -660478335);
    b = G(b, c, d, a, m[_i + 4], 20, -405537848);
    a = G(a, b, c, d, m[_i + 9], 5, 568446438);
    d = G(d, a, b, c, m[_i + 14], 9, -1019803690);
    c = G(c, d, a, b, m[_i + 3], 14, -187363961);
    b = G(b, c, d, a, m[_i + 8], 20, 1163531501);
    a = G(a, b, c, d, m[_i + 13], 5, -1444681467);
    d = G(d, a, b, c, m[_i + 2], 9, -51403784);
    c = G(c, d, a, b, m[_i + 7], 14, 1735328473);
    b = G(b, c, d, a, m[_i + 12], 20, -1926607734);
    a = H(a, b, c, d, m[_i + 5], 4, -378558);
    d = H(d, a, b, c, m[_i + 8], 11, -2022574463);
    c = H(c, d, a, b, m[_i + 11], 16, 1839030562);
    b = H(b, c, d, a, m[_i + 14], 23, -35309556);
    a = H(a, b, c, d, m[_i + 1], 4, -1530992060);
    d = H(d, a, b, c, m[_i + 4], 11, 1272893353);
    c = H(c, d, a, b, m[_i + 7], 16, -155497632);
    b = H(b, c, d, a, m[_i + 10], 23, -1094730640);
    a = H(a, b, c, d, m[_i + 13], 4, 681279174);
    d = H(d, a, b, c, m[_i + 0], 11, -358537222);
    c = H(c, d, a, b, m[_i + 3], 16, -722521979);
    b = H(b, c, d, a, m[_i + 6], 23, 76029189);
    a = H(a, b, c, d, m[_i + 9], 4, -640364487);
    d = H(d, a, b, c, m[_i + 12], 11, -421815835);
    c = H(c, d, a, b, m[_i + 15], 16, 530742520);
    b = H(b, c, d, a, m[_i + 2], 23, -995338651);
    a = I(a, b, c, d, m[_i + 0], 6, -198630844);
    d = I(d, a, b, c, m[_i + 7], 10, 1126891415);
    c = I(c, d, a, b, m[_i + 14], 15, -1416354905);
    b = I(b, c, d, a, m[_i + 5], 21, -57434055);
    a = I(a, b, c, d, m[_i + 12], 6, 1700485571);
    d = I(d, a, b, c, m[_i + 3], 10, -1894986606);
    c = I(c, d, a, b, m[_i + 10], 15, -1051523);
    b = I(b, c, d, a, m[_i + 1], 21, -2054922799);
    a = I(a, b, c, d, m[_i + 8], 6, 1873313359);
    d = I(d, a, b, c, m[_i + 15], 10, -30611744);
    c = I(c, d, a, b, m[_i + 6], 15, -1560198380);
    b = I(b, c, d, a, m[_i + 13], 21, 1309151649);
    a = I(a, b, c, d, m[_i + 4], 6, -145523070);
    d = I(d, a, b, c, m[_i + 11], 10, -1120210379);
    c = I(c, d, a, b, m[_i + 2], 15, 718787259);
    b = I(b, c, d, a, m[_i + 9], 21, -343485551);
    a = a + aa >>> 0;
    b = b + bb >>> 0;
    c = c + cc >>> 0;
    d = d + dd >>> 0;
  }

  return (0, _bytesToHex["default"])((0, _wordsToBytes["default"])((0, _swapEndian["default"])([a, b, c, d])));
}

module.exports = exports.default;