"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = stringify;

var _isObject = _interopRequireDefault(require("../lang/isObject"));

var _stringify = _interopRequireDefault(require("../qs/stringify"));

function stringify(urlParts) {
  if (!(0, _isObject["default"])(urlParts)) {
    throw new TypeError('invalid url parts object');
  }

  var protocol = urlParts.protocol,
      username = urlParts.username,
      password = urlParts.password,
      host = urlParts.host,
      hostname = urlParts.hostname,
      _urlParts$port = urlParts.port,
      port = _urlParts$port === void 0 ? '' : _urlParts$port,
      _urlParts$pathname = urlParts.pathname,
      pathname = _urlParts$pathname === void 0 ? '' : _urlParts$pathname,
      _urlParts$search = urlParts.search,
      search = _urlParts$search === void 0 ? '' : _urlParts$search,
      _urlParts$query = urlParts.query,
      query = _urlParts$query === void 0 ? {} : _urlParts$query,
      _urlParts$hash = urlParts.hash,
      hash = _urlParts$hash === void 0 ? '' : _urlParts$hash;

  if (!protocol) {
    throw new TypeError('protocol missing');
  }

  if (!host && !hostname) {
    throw new TypeError('host or hostname missing');
  }

  var url = '';
  url += "".concat(protocol, "://");

  if (username) {
    url += username;

    if (password) {
      url += ":".concat(password);
    }

    url += '@';
  }

  if (host) {
    url += host;
  } else {
    url += hostname;

    if (port) {
      url += ":".concat(port);
    }
  }

  url += pathname;

  if (search) {
    url += search;
  } else {
    /* istanbul ignore else */
    if (query) {
      var _search = (0, _stringify["default"])(query);

      if (_search) {
        url += "?".concat(_search);
      }
    }
  }

  url += hash;
  return url;
}

module.exports = exports.default;