"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = uuid;

var _isBrowser = _interopRequireDefault(require("./_internal/isBrowser"));

/**
 * Generate a uuid from pseudo-random numbers
 *
 * Ref: https://tools.ietf.org/html/rfc4122
 */
var hexDigits = '0123456789abcdef';
var maxUint32 = Math.pow(2, 32) - 1;
var random = Math.random;

if (_isBrowser["default"]) {
  var getRandomValues;
  /* istanbul ignore else */

  if (window.crypto && typeof window.crypto.getRandomValues === 'function') {
    getRandomValues = function getRandomValues(array) {
      return window.crypto.getRandomValues(array);
    };
  } else {
    // @ts-ignore
    if (window.msCrypto && typeof window.msCrypto.getRandomValues === 'function') {
      // @ts-ignore
      getRandomValues = function getRandomValues(array) {
        return window.msCrypto.getRandomValues(array);
      };
    }
  }
  /* istanbul ignore else */
  // @ts-ignore


  if (typeof window.Uint32Array === 'function' && getRandomValues) {
    random = function random() {
      return getRandomValues(new Uint32Array(1))[0] / maxUint32;
    };
  }
}

function uuid() {
  var u = [];

  for (var i = 0; i < 36; i++) {
    u[i] = hexDigits.charAt(random() * 16);
  }

  u[8] = u[13] = u[18] = u[23] = '-';
  u[14] = '4';
  u[19] = (Number(u[19]) & 3 | 8).toString(16);
  return u.join('');
}

module.exports = exports.default;